@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2002-2007 The Cacti Group                                 |
+ | Copyright (C) 2002-2012 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -92,6 +92,9 @@ static const struct keyword actions[] = {
 	{ "SNMP",       POLLER_ACTION_SNMP               },
 	{ "SCRIPT",     POLLER_ACTION_SCRIPT             },
 	{ "PHPSCRIPT",	POLLER_ACTION_PHP_SCRIPT_SERVER  },
+	{ "SNMP_CT",        POLLER_ACTION_SNMP_COUNT               },
+	{ "SCRIPT_CT",      POLLER_ACTION_SCRIPT_COUNT             },
+	{ "PHPSCRIPT_CT",	POLLER_ACTION_PHP_SCRIPT_SERVER_COUNT  },
 
 	{ 0, 0 }	/* ENDMARKER */
 };
