@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2002-2007 The Cacti Group                                 |
+ | Copyright (C) 2002-2012 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -50,7 +50,7 @@
 
 DEFINE_SPINE_LOCK(snmp)
 DEFINE_SPINE_LOCK(thread)
-DEFINE_SPINE_LOCK(mysql)
+DEFINE_SPINE_LOCK(seteuid)
 DEFINE_SPINE_LOCK(ghbn)
 DEFINE_SPINE_LOCK(pipe)
 DEFINE_SPINE_LOCK(syslog)
@@ -69,7 +69,7 @@ DEFINE_SPINE_LOCK(php_proc_9)
 void init_mutexes() {
 	pthread_once((pthread_once_t*) get_attr(LOCK_SNMP_O),       init_snmp_lock);
 	pthread_once((pthread_once_t*) get_attr(LOCK_THREAD_O),     init_thread_lock);
-	pthread_once((pthread_once_t*) get_attr(LOCK_MYSQL_O),      init_mysql_lock);
+	pthread_once((pthread_once_t*) get_attr(LOCK_SETEUID_O),    init_seteuid_lock);
 	pthread_once((pthread_once_t*) get_attr(LOCK_GHBN_O),       init_ghbn_lock);
 	pthread_once((pthread_once_t*) get_attr(LOCK_PIPE_O),       init_pipe_lock);
 	pthread_once((pthread_once_t*) get_attr(LOCK_SYSLOG_O),     init_syslog_lock);
@@ -92,7 +92,7 @@ pthread_mutex_t* get_lock(int lock) {
 	switch (lock) {
 	case LOCK_SNMP:       ret_val = &snmp_lock;       break;
 	case LOCK_THREAD:     ret_val = &thread_lock;     break;
-	case LOCK_MYSQL:      ret_val = &mysql_lock;      break;
+	case LOCK_SETEUID:    ret_val = &seteuid_lock;    break;
 	case LOCK_GHBN:       ret_val = &ghbn_lock;       break;
 	case LOCK_PIPE:       ret_val = &pipe_lock;       break;
 	case LOCK_SYSLOG:     ret_val = &syslog_lock;     break;
@@ -118,7 +118,7 @@ pthread_once_t* get_attr(int locko) {
 	switch (locko) {
 	case LOCK_SNMP_O:       ret_val = &snmp_lock_o;       break;
 	case LOCK_THREAD_O:     ret_val = &thread_lock_o;     break;
-	case LOCK_MYSQL_O:      ret_val = &mysql_lock_o;      break;
+	case LOCK_SETEUID_O:    ret_val = &seteuid_lock_o;    break;
 	case LOCK_GHBN_O:       ret_val = &ghbn_lock_o;       break;
 	case LOCK_PIPE_O:       ret_val = &pipe_lock_o;       break;
 	case LOCK_SYSLOG_O:     ret_val = &syslog_lock_o;     break;
