@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2002-2007 The Cacti Group                                 |
+ | Copyright (C) 2002-2013 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -53,27 +53,20 @@ int ping_host(host_t *host, ping_t *ping) {
 	snmp_result = 0;
 
 	/* icmp/tcp/udp ping test */
-	if ((host->availability_method == AVAIL_SNMP_AND_PING) || (host->availability_method == AVAIL_PING)) {
-		if (host->ping_method == PING_ICMP) {
-			/* set and then test for asroot */
-			#ifndef __CYGWIN__
-			seteuid(0);
+	if ((host->availability_method == AVAIL_SNMP_AND_PING) ||
+		(host->availability_method == AVAIL_PING) ||
+		(host->availability_method == AVAIL_SNMP_OR_PING)) {
 
-			if (geteuid() != 0) {
+		if (host->ping_method == PING_ICMP) {
+			if (set.icmp_avail == FALSE) {
+				SPINE_LOG_DEBUG(("Host[%i] DEBUG Falling back to UDP Ping Due to SetUID Issues", host->id));
 				host->ping_method = PING_UDP;
-				SPINE_LOG_DEBUG(("WARNING: Falling back to UDP Ping due to not running asroot.  Please use \"chmod xxx0 /usr/bin/spine\" to resolve.\n"));
 			}
-			#endif
 		}
 
 		if (!strstr(host->hostname, "localhost")) {
 			if (host->ping_method == PING_ICMP) {
 				ping_result = ping_icmp(host, ping);
-
-				/* give up root privileges */
-				#ifndef __CYGWIN__
-				seteuid(getuid());
-				#endif
 			}else if (host->ping_method == PING_UDP) {
 				ping_result = ping_udp(host, ping);
 			}else if (host->ping_method == PING_TCP) {
@@ -88,13 +81,30 @@ int ping_host(host_t *host, ping_t *ping) {
 
 	/* snmp test */
 	if ((host->availability_method == AVAIL_SNMP) ||
-		((host->availability_method == AVAIL_SNMP_AND_PING) && (ping_result != HOST_UP))) {
+		(host->availability_method == AVAIL_SNMP_GET_SYSDESC) ||
+		(host->availability_method == AVAIL_SNMP_GET_NEXT) ||
+		(host->availability_method == AVAIL_SNMP_AND_PING) ||
+		((host->availability_method == AVAIL_SNMP_OR_PING) && (ping_result != HOST_UP))) {
 		snmp_result = ping_snmp(host, ping);
 	}
 
 	switch (host->availability_method) {
 		case AVAIL_SNMP_AND_PING:
-			if (strlen(host->snmp_community) == 0) {
+			if ((strlen(host->snmp_community) == 0) && (host->snmp_version < 3)) {
+				if (ping_result == HOST_UP) {
+					return HOST_UP;
+				}else{
+					return HOST_DOWN;
+				}
+			}
+
+			if ((snmp_result == HOST_UP) && (ping_result == HOST_UP)) {
+				return HOST_UP;
+			}else{
+				return HOST_DOWN;
+			}
+		case AVAIL_SNMP_OR_PING:
+			if ((strlen(host->snmp_community) == 0) && (host->snmp_version < 3)) {
 				if (ping_result == HOST_UP) {
 					return HOST_UP;
 				}else{
@@ -112,6 +122,8 @@ int ping_host(host_t *host, ping_t *ping) {
 				return HOST_DOWN;
 			}
 		case AVAIL_SNMP:
+		case AVAIL_SNMP_GET_NEXT:
+		case AVAIL_SNMP_GET_SYSDESC:
 			if (snmp_result == HOST_UP) {
 				return HOST_UP;
 			}else{
@@ -148,57 +160,63 @@ int ping_snmp(host_t *host, ping_t *ping) {
 	double begin_time, end_time, total_time;
 	double one_thousand = 1000.00;
 
-	if (strlen(host->snmp_community) != 0) {
-		/* by default, we look at sysUptime */
-		if ((oid = strdup(".1")) == NULL) {
-			die("ERROR: malloc(): strdup() oid ping.c failed");
-		}
+	SPINE_LOG_DEBUG(("Host[%i] DEBUG: Entering SNMP Ping", host->id));
 
-		/* record start time */
-		retry:
-		begin_time = get_time_as_double();
+	if (host->snmp_session) {
+		if ((strlen(host->snmp_community) != 0) || (host->snmp_version == 3)) {
+			/* by default, we look at sysUptime */
+			if (host->availability_method == AVAIL_SNMP_GET_NEXT) {
+				oid = strdup(".1.3");
+			}else if (host->availability_method == AVAIL_SNMP_GET_SYSDESC) {
+				oid = strdup(".1.3.6.1.2.1.1.1.0");
+			}else {
+				oid = strdup(".1.3.6.1.2.1.1.3.0");
+			}
 
-		poll_result = snmp_getnext(host, oid);
+			if (oid == NULL) die("ERROR: malloc(): strdup() oid ping.c failed");
 
-		/* record end time */
-		end_time = get_time_as_double();
+			/* record start time */
+			begin_time = get_time_as_double();
 
-		free(oid);
+			if (host->availability_method == AVAIL_SNMP_GET_NEXT) {
+				poll_result = snmp_getnext(host, oid);
+			} else {
+				poll_result = snmp_get(host, oid);
+			}
 
-		total_time = (end_time - begin_time) * one_thousand;
+			/* record end time */
+			end_time = get_time_as_double();
 
-		if ((strlen(poll_result) == 0) || IS_UNDEFINED(poll_result)) {
-			if (num_oids_checked > 1) {
-				if (num_oids_checked == 0) {
-					/* use sysUptime as a backup if the generic OID fails */
-					if ((oid = strdup(".1.3.6.1.2.1.1.3.0")) == NULL) {
-						die("ERROR: malloc(): strdup() oid ping.c failed");
-					}
-				}else{
-					/* use sysDescription as a backup if sysUptime fails */
-					if ((oid = strdup(".1.3.6.1.2.1.1.1.0")) == NULL) {
-						die("ERROR: malloc(): strdup() oid ping.c failed");
-					}
-				}
+			free(oid);
 
+			total_time = (end_time - begin_time) * one_thousand;
+
+			/* do positive test cases first */
+			if (host->snmp_status == SNMPERR_UNKNOWN_OBJID) {
+				snprintf(ping->snmp_response, SMALL_BUFSIZE, "Host responded to SNMP");
+				snprintf(ping->snmp_status, 50, "%.5f", total_time);
 				free(poll_result);
-				num_oids_checked++;
-				goto retry;
-			}else{
+				return HOST_UP;
+			}else if (host->snmp_status != SNMPERR_SUCCESS) {
+				SPINE_LOG_MEDIUM(("Host[%i] SNMP Ping Error: %s", host->id, snmp_api_errstring(host->snmp_status)));
 				snprintf(ping->snmp_response, SMALL_BUFSIZE, "Host did not respond to SNMP");
 				free(poll_result);
 				return HOST_DOWN;
+			}else{
+				snprintf(ping->snmp_response, SMALL_BUFSIZE, "Host responded to SNMP");
+				snprintf(ping->snmp_status, 50, "%.5f", total_time);
+				free(poll_result);
+				return HOST_UP;
 			}
 		}else{
-			snprintf(ping->snmp_response, SMALL_BUFSIZE, "Host responded to SNMP");
-			snprintf(ping->snmp_status, 50, "%.5f", total_time);
-			free(poll_result);
+			snprintf(ping->snmp_status, 50, "0.00");
+			snprintf(ping->snmp_response, SMALL_BUFSIZE, "Host does not require SNMP");
 			return HOST_UP;
 		}
 	}else{
 		snprintf(ping->snmp_status, 50, "0.00");
-		snprintf(ping->snmp_response, SMALL_BUFSIZE, "Host does not require SNMP");
-		return HOST_UP;
+		snprintf(ping->snmp_response, SMALL_BUFSIZE, "Invalid SNMP Session");
+		return HOST_DOWN;
 	}
 }
 
@@ -226,10 +244,10 @@ int ping_icmp(host_t *host, ping_t *ping) {
 	struct sockaddr_in fromname;
 	char   socket_reply[BUFSIZE];
 	int    retry_count;
-	char   *cacti_msg = "cacti-monitoring-system";
+	char   *cacti_msg = "cacti-monitoring-system\0";
 	int    packet_len;
-	int    fromlen;
-	int    return_code;
+	socklen_t    fromlen;
+	ssize_t    return_code;
 	fd_set socket_fds;
 
 	static   unsigned int seq = 0;
@@ -239,13 +257,50 @@ int ping_icmp(host_t *host, ping_t *ping) {
 	unsigned char  *packet;
 	char     *new_hostname;
 
+	SPINE_LOG_DEBUG(("Host[%i] DEBUG: Entering ICMP Ping", host->id));
+
 	/* remove "tcp:" from hostname */
 	new_hostname = remove_tcp_udp_from_hostname(host->hostname);
 
 	/* get ICMP socket */
-	if ((icmp_socket = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) == -1) {
-		SPINE_LOG(("ERROR: ping_icmp: cannot open an ICMP socket\n"));
+	retry_count = 0;
+	while ( TRUE ) {
+		#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
+		if (hasCaps() != TRUE) {
+			thread_mutex_lock(LOCK_SETEUID);
+			seteuid(0);
+		}
+		#endif
+
+		if ((icmp_socket = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) == -1) {
+			usleep(500000);
+			retry_count++;
+
+			if (retry_count > 4) {
+				snprintf(ping->ping_response, SMALL_BUFSIZE, "ICMP: Ping unable to create ICMP Socket");
+				snprintf(ping->ping_status, 50, "down");
+				free(new_hostname);
+				#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
+				if (hasCaps() != TRUE) {
+					seteuid(getuid());
+					thread_mutex_unlock(LOCK_SETEUID);
+				}
+				#endif
+
+				return HOST_DOWN;
+	
+				break;
+			}
+		}else{
+			break;
+		}
 	}
+	#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
+	if (hasCaps() != TRUE) {
+		seteuid(getuid());
+		thread_mutex_unlock(LOCK_SETEUID);
+	}
+	#endif
 
 	/* convert the host timeout to a double precision number in seconds */
 	host_timeout = host->ping_timeout;
@@ -272,10 +327,6 @@ int ping_icmp(host_t *host, ping_t *ping) {
 	thread_mutex_lock(LOCK_GHBN);
 	icmp->icmp_seq = seq++;
 	thread_mutex_unlock(LOCK_GHBN);
-	
-	if (gettimeofday((struct timeval*)(icmp+1), NULL) == -1) {
-		die("ERROR: Function gettimeofday failed.  Exiting spine");
-	}
 
 	icmp->icmp_cksum = 0;
 	memcpy(packet+ICMP_HDR_SIZE, cacti_msg, strlen(cacti_msg));
@@ -291,6 +342,7 @@ int ping_icmp(host_t *host, ping_t *ping) {
 		if (init_sockaddr(&fromname, new_hostname, 7)) {
 			retry_count = 0;
 			total_time  = 0;
+			begin_time  = 0;
 
 			/* initialize file descriptor to review for input/output */
 			FD_ZERO(&socket_fds);
@@ -307,7 +359,7 @@ int ping_icmp(host_t *host, ping_t *ping) {
 				}
 
 				/* record start time */
-				if (total_time == 0) { 
+				if (total_time == 0) {
 					/* establish timeout value */
 					timeout.tv_sec  = 0;
 					timeout.tv_usec = host->ping_timeout * 1000;
@@ -325,9 +377,10 @@ int ping_icmp(host_t *host, ping_t *ping) {
 				/* send packet to destination */
 				return_code = sendto(icmp_socket, packet, packet_len, 0, (struct sockaddr *) &fromname, sizeof(fromname));
 
-	   			fromlen = sizeof(fromname);
+				fromlen = sizeof(fromname);
 
 				/* wait for a response on the socket */
+				keep_listening:
 				return_code = select(FD_SETSIZE, &socket_fds, NULL, NULL, &timeout);
 
 				/* record end time */
@@ -338,25 +391,43 @@ int ping_icmp(host_t *host, ping_t *ping) {
 
 				/* check to see which socket talked */
 				if (total_time < host_timeout) {
+					#if !(defined(__CYGWIN__))
 					return_code = recvfrom(icmp_socket, socket_reply, BUFSIZE, MSG_WAITALL, (struct sockaddr *) &recvname, &fromlen);
+					#else
+					return_code = recvfrom(icmp_socket, socket_reply, BUFSIZE, MSG_PEEK, (struct sockaddr *) &recvname, &fromlen);
+					#endif
 
 					if (return_code < 0) {
 						if (errno == EINTR) {
+							SPINE_LOG_DEBUG(("Host[%i] DEBUG: Received EINTR", host->id));
 							/* call was interrupted by some system event */
-							continue;
+							goto keep_listening;
 						}
 					}else{
-						ip = (struct ip *) socket_reply;
-	
-						pkt   = (struct icmp *)  (socket_reply + (ip->ip_hl << 2));
+						ip  = (struct ip *) socket_reply;
+						pkt = (struct icmp *)  (socket_reply + (ip->ip_hl << 2));
 
 						if (fromname.sin_addr.s_addr == recvname.sin_addr.s_addr) {
 							if ((pkt->icmp_type == ICMP_ECHOREPLY)) {
+								SPINE_LOG_DEBUG(("Host[%i] DEBUG: ICMP Host Alive, Try Count:%i, Time:%.4f ms", host->id, retry_count+1, (total_time)));
 								snprintf(ping->ping_response, SMALL_BUFSIZE, "ICMP: Host is Alive");
 								snprintf(ping->ping_status, 50, "%.5f", total_time);
 								free(new_hostname);
 								free(packet);
+								#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
+								if (hasCaps() != TRUE) {
+									thread_mutex_lock(LOCK_SETEUID);
+									seteuid(0);
+								}
+								#endif
 								close(icmp_socket);
+								#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
+								if (hasCaps() != TRUE) {
+									seteuid(getuid());
+									thread_mutex_unlock(LOCK_SETEUID);
+								}
+								#endif
+
 								return HOST_UP;
 							}else{
 								/* received a response other than an echo reply */
@@ -369,14 +440,11 @@ int ping_icmp(host_t *host, ping_t *ping) {
 							}
 						}else{
 							/* another host responded */
-							if (total_time > host_timeout) {
-								retry_count++;
-								total_time = 0;
-							}
-
-							continue;
+							goto keep_listening;
 						}
 					}
+				}else{
+					SPINE_LOG_DEBUG(("Host[%i] DEBUG: Exceeded Host Timeout, Retrying", host->id));
 				}
 
 				total_time = 0;
@@ -390,7 +458,19 @@ int ping_icmp(host_t *host, ping_t *ping) {
 			snprintf(ping->ping_status, 50, "down");
 			free(new_hostname);
 			free(packet);
+			#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
+			if (hasCaps() != TRUE) {
+				thread_mutex_lock(LOCK_SETEUID);
+				seteuid(0);
+			}
+			#endif
 			close(icmp_socket);
+			#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
+			if (hasCaps() != TRUE) {
+				seteuid(getuid());
+				thread_mutex_unlock(LOCK_SETEUID);
+			}
+			#endif
 			return HOST_DOWN;
 		}
 	}else{
@@ -398,7 +478,21 @@ int ping_icmp(host_t *host, ping_t *ping) {
 		snprintf(ping->ping_status, 50, "down");
 		free(new_hostname);
 		free(packet);
-  		if (icmp_socket != -1) close(icmp_socket);
+		if (icmp_socket != -1) {
+			#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
+			if (hasCaps() != TRUE) {
+				thread_mutex_lock(LOCK_SETEUID);
+				seteuid(0);
+			}
+			#endif
+			close(icmp_socket);
+			#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
+			if (hasCaps() != TRUE) {
+				seteuid(getuid());
+				thread_mutex_unlock(LOCK_SETEUID);
+			}
+			#endif
+		}
 		return HOST_DOWN;
 	}
 }
@@ -430,13 +524,14 @@ int ping_udp(host_t *host, ping_t *ping) {
 	fd_set socket_fds;
 	char   *new_hostname;
 
+	SPINE_LOG_DEBUG(("Host[%i] DEBUG: Entering UDP Ping", host->id));
+
+	/* set total time */
+	total_time = 0;
+
 	/* remove "udp:" from hostname */
 	new_hostname = remove_tcp_udp_from_hostname(host->hostname);
 
-	/* establish timeout value */
-	timeout.tv_sec  = 0;
-	timeout.tv_usec = host->ping_timeout * 1000;
-
 	/* convert the host timeout to a double precision number in seconds */
 	host_timeout = host->ping_timeout;
 
@@ -451,6 +546,7 @@ int ping_udp(host_t *host, ping_t *ping) {
 
 		/* set the socket timeout */
 		setsockopt(udp_socket, SOL_SOCKET, SO_RCVTIMEO, (char*)&timeout, sizeof(timeout));
+		setsockopt(udp_socket, SOL_SOCKET, SO_SNDTIMEO, (char*)&timeout, sizeof(timeout));
 
 		/* get address of hostname */
 		if (init_sockaddr(&servername, new_hostname, host->ping_port)) {
@@ -484,37 +580,59 @@ int ping_udp(host_t *host, ping_t *ping) {
 				/* record start time */
 				begin_time = get_time_as_double();
 
+				/* establish timeout value */
+				if (host->ping_timeout >= 1000) {
+					timeout.tv_sec  = rint(floor(host_timeout / 1000));
+					timeout.tv_usec = (timeout.tv_sec * 1000000) - (host->ping_timeout * 1000);
+				}else{
+					timeout.tv_sec  = 0;
+					timeout.tv_usec = (host->ping_timeout * 1000);
+				}
+
 				/* send packet to destination */
 				send(udp_socket, request, request_len, 0);
 
 				/* wait for a response on the socket */
-				select(FD_SETSIZE, &socket_fds, NULL, NULL, &timeout);
+				wait_more:
+				return_code = select(FD_SETSIZE, &socket_fds, NULL, NULL, &timeout);
 
 				/* record end time */
 				end_time = get_time_as_double();
 
-				/* check to see which socket talked */
-				if (FD_ISSET(udp_socket, &socket_fds)) {
-					return_code = read(udp_socket, socket_reply, BUFSIZE);
-				}else{
-					return_code = -10;
-				}
-
 				/* caculate total time */
 				total_time = (end_time - begin_time) * one_thousand;
 
-				SPINE_LOG_DEBUG(("DEBUG: UDP Ping return_code was %i, errno was %i, total_time was %.4f\n", return_code, errno, (total_time*1000)));
+				/* check to see which socket talked */
+				if (return_code > 0) {
+					if (FD_ISSET(udp_socket, &socket_fds)) {
+						return_code = read(udp_socket, socket_reply, BUFSIZE);
 
-				if ((return_code >= 0) || ((return_code == -1) && ((errno == ECONNRESET) || (errno == ECONNREFUSED)))) {
-					if (total_time <= host_timeout) {
-						snprintf(ping->ping_response, SMALL_BUFSIZE, "UDP: Host is Alive");
+						if ((return_code == -1) && ((errno == ECONNRESET) || (errno == ECONNREFUSED))) {
+							SPINE_LOG_DEBUG(("Host[%i] DEBUG: UDP Host Alive, Try Count:%i, Time:%.4f ms", host->id, retry_count+1, (total_time)));
+							snprintf(ping->ping_response, SMALL_BUFSIZE, "UDP: Host is Alive");
+							snprintf(ping->ping_status, 50, "%.5f", total_time);
+							free(new_hostname);
+							close(udp_socket);
+							return HOST_UP;
+						}
+					}
+				}else if (return_code == -1) {
+					if (errno == EINTR) {
+						/* interrupted, try again */
+						goto wait_more;
+					}else{
+						snprintf(ping->ping_response, SMALL_BUFSIZE, "UDP: Host is Down");
 						snprintf(ping->ping_status, 50, "%.5f", total_time);
 						free(new_hostname);
 						close(udp_socket);
-						return HOST_UP;
+						return HOST_DOWN;
 					}
+				}else{
+					/* timeout */
 				}
 
+				SPINE_LOG_DEBUG(("Host[%i] DEBUG: UDP Timeout, Try Count:%i, Time:%.4f ms", host->id, retry_count+1, (total_time)));
+
 				retry_count++;
 				#ifndef SOLAR_THREAD
 				usleep(1000);
@@ -557,22 +675,26 @@ int ping_tcp(host_t *host, ping_t *ping) {
 	int    tcp_socket;
 	struct sockaddr_in servername;
 	int    retry_count;
-	char   request[BUFSIZE];
-	int    request_len;
 	int    return_code;
-	fd_set socket_fds;
 	char   *new_hostname;
 
+	SPINE_LOG_DEBUG(("Host[%i] DEBUG: Entering TCP Ping", host->id));
+
 	/* remove "tcp:" from hostname */
 	new_hostname = remove_tcp_udp_from_hostname(host->hostname);
 
-	/* establish timeout value */
-	timeout.tv_sec  = 0;
-	timeout.tv_usec = host->ping_timeout * 1000;
-
 	/* convert the host timeout to a double precision number in seconds */
 	host_timeout = host->ping_timeout;
 
+	/* establish timeout value */
+	if (host->ping_timeout >= 1000) {
+		timeout.tv_sec  = rint(floor(host_timeout / 1000));
+		timeout.tv_usec = (timeout.tv_sec * 1000000) - (host->ping_timeout * 1000);
+	}else{
+		timeout.tv_sec  = 0;
+		timeout.tv_usec = (host->ping_timeout * 1000);
+	}
+
 	/* initilize the socket */
 	tcp_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
 
@@ -592,9 +714,35 @@ int ping_tcp(host_t *host, ping_t *ping) {
 			retry_count = 0;
 
 			while (1) {
+				/* record start time */
+				begin_time  = get_time_as_double();
+
+				/* make the connection */
 				return_code = connect(tcp_socket, (struct sockaddr *) &servername, sizeof(servername));
-				if (return_code < 0) {
-					if (retry_count > host->ping_retries) { 
+
+				/* record end time */
+				end_time = get_time_as_double();
+
+				/* caculate total time */
+				total_time = (end_time - begin_time) * one_thousand;
+
+				if (((return_code == -1) && (errno == ECONNREFUSED)) ||
+					(return_code == 0)) {
+					SPINE_LOG_DEBUG(("Host[%i] DEBUG: TCP Host Alive, Try Count:%i, Time:%.4f ms", host->id, retry_count+1, (total_time)));
+					snprintf(ping->ping_response, SMALL_BUFSIZE, "TCP: Host is Alive");
+					snprintf(ping->ping_status, 50, "%.5f", total_time);
+					free(new_hostname);
+					close(tcp_socket);
+					return HOST_UP;
+				}else{
+					#if defined(__CYGWIN__)
+					snprintf(ping->ping_status, 50, "down");
+					snprintf(ping->ping_response, SMALL_BUFSIZE, "TCP: Cannot connect to host");
+					free(new_hostname);
+					close(tcp_socket);
+					return HOST_DOWN;
+					#else
+					if (retry_count > host->ping_retries) {
 						snprintf(ping->ping_status, 50, "down");
 						snprintf(ping->ping_response, SMALL_BUFSIZE, "TCP: Cannot connect to host");
 						free(new_hostname);
@@ -603,59 +751,9 @@ int ping_tcp(host_t *host, ping_t *ping) {
 					}else{
 						retry_count++;
 					}
-				}else{
-					break;
+					#endif
 				}
 			}
-
-			/* format packet */
-			snprintf(request, BUFSIZE, "cacti-monitoring-system"); /* the actual test data */
-			request_len = strlen(request);
-
-			retry_count = 0;
-
-			/* initialize file descriptor to review for input/output */
-			FD_ZERO(&socket_fds);
-			FD_SET(tcp_socket,&socket_fds);
-
-			while (1) {
-				if (retry_count > host->ping_retries) {
-					snprintf(ping->ping_response, SMALL_BUFSIZE, "TCP: Ping timed out");
-					snprintf(ping->ping_status, 50, "down");
-					free(new_hostname);
-					close(tcp_socket);
-					return HOST_DOWN;
-				}
-
-				/* record start time */
-				begin_time = get_time_as_double();
-
-				/* send packet to destination */
-				return_code = send(tcp_socket, request, request_len, 0);
-
-				/* record end time */
-				end_time = get_time_as_double();
-
-				/* caculate total time */
-				total_time = (end_time - begin_time) * one_thousand;
-
-				SPINE_LOG_DEBUG(("DEBUG: TCP Ping return_code was %i, errno was %i, total_time was %.4f\n", return_code, errno, (total_time*1000)));
-
-				if (return_code >= 0) {
-					if (total_time <= host_timeout) {
-						snprintf(ping->ping_response, SMALL_BUFSIZE, "TCP: Host is Alive");
-						snprintf(ping->ping_status, 50, "%.5f", total_time);
-						free(new_hostname);
-						close(tcp_socket);
-						return HOST_UP;
-					}
-				}
-
-				retry_count++;
-				#ifndef SOLAR_THREAD
-				usleep(1000);
-				#endif
-			}
 		}else{
 			snprintf(ping->ping_response, SMALL_BUFSIZE, "TCP: Destination hostname invalid");
 			snprintf(ping->ping_status, 50, "down");
@@ -680,33 +778,142 @@ int ping_tcp(host_t *host, ping_t *ping) {
  */
 int init_sockaddr(struct sockaddr_in *name, const char *hostname, unsigned short int port) {
 	struct hostent *hostinfo;
-	int    i;
+	int retry_count;
+	#if !defined(H_ERRNO_DECLARED) && !defined(_AIX)
+	extern int h_errno;
+	#endif
 
 	name->sin_family = AF_INET;
 	name->sin_port   = htons (port);
 
-	/* retry 3 times to contact host */
-	i = 0;
-	while (1) {
-		thread_mutex_lock(LOCK_GHBN);
-		hostinfo = gethostbyname(hostname);
-		if (hostinfo == NULL) {
-			SPINE_LOG(("WARNING: Unknown host %s\n", hostname));
+	retry_count = 0;
 
-			if (i > 3) {
-				thread_mutex_unlock(LOCK_GHBN);
+	#ifdef HAVE_THREADSAFE_GETHOSTBYNAME
+	retry:
+	hostinfo = gethostbyname(hostname);
+
+	if (!hostinfo) {
+		if (h_errno == TRY_AGAIN && retry_count < 3) {
+			retry_count++;
+			usleep(50000);
+			goto retry;
+		}else{
+			return NULL;
+		}
+	}else{
+		name->sin_addr = *(struct in_addr *) hostinfo->h_addr;
+	}
+
+	#else
+	#ifdef HAVE_GETHOSTBYNAME_R_GLIBC
+	struct hostent result_buf;
+	size_t len = 1024;
+	char   *buf;
+	int    herr;
+	int    rv;
+
+	buf = malloc(len*sizeof(char));
+	memset(buf, 0, sizeof(buf));
+
+	while (1) {
+		rv = gethostbyname_r(hostname, &result_buf, buf, len,
+		&hostinfo, &herr);
+
+		if (!hostinfo) {
+			if (rv == ERANGE) {
+				len *= 2;
+				buf = realloc(buf, len*sizeof(char));
+
+				continue;
+			}else if (herr == TRY_AGAIN && retry_count < 3) {
+				retry_count++;
+				usleep(50000);
+				continue;
+			}else{
+				free(buf);
 				return FALSE;
 			}
-			i++;
-			#ifndef SOLAR_THREAD
-			usleep(1000);
-			#endif
 		}else{
-			name->sin_addr = *(struct in_addr *) hostinfo->h_addr;
-			thread_mutex_unlock(LOCK_GHBN);
-			return TRUE;
+			break;
 		}
 	}
+
+	name->sin_addr = *(struct in_addr *) hostinfo->h_addr;
+
+	free(buf);
+	#else
+	#ifdef HAVE_GETHOSTBYNAME_R_SOLARIS
+	size_t  len = 8192;
+	char   *buf = NULL;
+	struct hostent result;
+
+	buf = malloc(len*sizeof(char));
+	memset(buf, 0, sizeof(buf));
+
+	while (1) {
+		hostinfo = gethostbyname_r(hostname, &result, buf, len, &h_errno);
+		if (!hostinfo) {
+			if (errno == ERANGE) {
+				len += 1024;
+				buf = realloc(buf, len*sizeof(char));
+				memset(buf, 0, sizeof(buf));
+
+				continue;
+			}else if (h_errno == TRY_AGAIN && retry_count < 3) {
+				retry_count++;
+				usleep(50000);
+				continue;
+			}else{
+				free(buf);
+				return NULL;
+			}
+		}else{
+			break;
+		}
+	}
+
+	name->sin_addr = *(struct in_addr *) hostinfo->h_addr;
+
+	free(buf);
+	#else
+	#ifdef HAVE_GETHOSTBYNAME_R_HPUX
+	struct hostent hostent;
+	struct hostent_data buf;
+	int rv;
+
+	rv = gethostbyname_r(hostname, &hostent, &buf);
+	if (!rv) {
+		name->sin_addr = *(struct in_addr *) hostent->h_addr;
+	}
+
+	#else
+	retry:
+	thread_mutex_lock(LOCK_GHBN);
+	hostinfo = gethostbyname(hostname);
+	if (!hostinfo) {
+		thread_mutex_unlock(LOCK_GHBN);
+		if (h_errno == TRY_AGAIN && retry_count < 3) {
+			retry_count++;
+			usleep(50000);
+			goto retry;
+		}else{
+			hostinfo = NULL;
+		}
+	}else{
+		name->sin_addr = *(struct in_addr *) hostinfo->h_addr;
+		thread_mutex_unlock(LOCK_GHBN);
+	}
+	#endif
+	#endif
+	#endif
+	#endif
+
+	if (hostinfo == NULL) {
+		SPINE_LOG(("WARNING: Unknown host %s", hostname));
+		return FALSE;
+	}else{
+		return TRUE;
+	}
 }
 
 /*! \fn char *remove_tcp_udp_from_hostname(char *hostname)
@@ -733,7 +940,7 @@ char *remove_tcp_udp_from_hostname(char *hostname) {
 	return(cleaned_hostname);
 }
 
-/*! \fn unsigned short get_checksum(void* buf, int len)
+/*! \fn unsigned short int get_checksum(void* buf, int len)
  *  \brief calculates a 16bit checksum of a packet buffer
  *  \param buf the input buffer to calculate the checksum of
  *  \param len the size of the input buffer
@@ -741,21 +948,27 @@ char *remove_tcp_udp_from_hostname(char *hostname) {
  *  \return 16bit checksum of an input buffer of size len.
  *
  */
-unsigned short get_checksum(void* buf, int len) {
-	int nleft = len;
-	int sum = 0;
-	unsigned short answer;
-	unsigned short* w = (unsigned short*)buf;
+unsigned short int get_checksum(void* buf, int len) {
+	int      nleft = len;
+	int32_t  sum   = 0;
+	unsigned short int answer;
+	unsigned short int* w = (unsigned short int*)buf;
+	unsigned short int odd_byte = 0;
 
 	while (nleft > 1) {
 		sum += *w++;
 		nleft -= 2;
 	}
-	if (nleft == 1)
-		sum += *(unsigned char*)w;
-	sum = (sum >> 16) + (sum & 0xffff);
-	sum += (sum >> 16);
+
+	if (nleft == 1) {
+   		*(unsigned char*)(&odd_byte) = *(unsigned char*)w;
+   		sum += odd_byte;
+	}
+
+	sum    = (sum >> 16) + (sum & 0xffff);
+	sum   += (sum >> 16);
 	answer = ~sum;				/* truncate to 16 bits */
+
 	return answer;
 }
 
@@ -799,15 +1012,16 @@ void update_host_status(int status, host_t *host, ping_t *ping, int availability
 
 		/*determine the error message to display */
 		switch (availability_method) {
+		case AVAIL_SNMP_OR_PING:
 		case AVAIL_SNMP_AND_PING:
-			if (strlen(host->snmp_community) == 0) {
+			if ((strlen(host->snmp_community) == 0) && (host->snmp_version < 3)) {
 				snprintf(host->status_last_error, SMALL_BUFSIZE, "%s", ping->ping_response);
 			}else {
 				snprintf(host->status_last_error, SMALL_BUFSIZE,"%s, %s",ping->snmp_response,ping->ping_response);
 			}
 			break;
 		case AVAIL_SNMP:
-			if (strlen(host->snmp_community) == 0) {
+			if ((strlen(host->snmp_community) == 0) && (host->snmp_version < 3)) {
 				snprintf(host->status_last_error, SMALL_BUFSIZE, "%s", "Device does not require SNMP");
 			}else {
 				snprintf(host->status_last_error, SMALL_BUFSIZE, "%s", ping->snmp_response);
@@ -935,29 +1149,32 @@ void update_host_status(int status, host_t *host, ping_t *ping, int availability
 		if ((host->status == HOST_UP) || (host->status == HOST_RECOVERING)) {
 			/* log ping result if we are to use a ping for reachability testing */
 			if (availability_method == AVAIL_SNMP_AND_PING) {
-				SPINE_LOG_HIGH(("Host[%i] PING Result: %s\n", host->id, ping->ping_response));
-				SPINE_LOG_HIGH(("Host[%i] SNMP Result: %s\n", host->id, ping->snmp_response));
+				SPINE_LOG_HIGH(("Host[%i] PING Result: %s", host->id, ping->ping_response));
+				SPINE_LOG_HIGH(("Host[%i] SNMP Result: %s", host->id, ping->snmp_response));
+			}else if (availability_method == AVAIL_SNMP_OR_PING) {
+				SPINE_LOG_HIGH(("Host[%i] PING Result: %s", host->id, ping->ping_response));
+				SPINE_LOG_HIGH(("Host[%i] SNMP Result: %s", host->id, ping->snmp_response));
 			}else if (availability_method == AVAIL_SNMP) {
-				if (strlen(host->snmp_community) == 0) {
-					SPINE_LOG_HIGH(("Host[%i] SNMP Result: Device does not require SNMP\n", host->id));
+				if ((strlen(host->snmp_community) == 0) && (host->snmp_version < 3)) {
+					SPINE_LOG_HIGH(("Host[%i] SNMP Result: Device does not require SNMP", host->id));
 				}else{
-					SPINE_LOG_HIGH(("Host[%i] SNMP Result: %s\n", host->id, ping->snmp_response));
+					SPINE_LOG_HIGH(("Host[%i] SNMP Result: %s", host->id, ping->snmp_response));
 				}
 			}else if (availability_method == AVAIL_NONE) {
-				SPINE_LOG_HIGH(("Host[%i] No Host Availability Method Selected\n", host->id));
+				SPINE_LOG_HIGH(("Host[%i] No Host Availability Method Selected", host->id));
 			}else{
-				SPINE_LOG_HIGH(("Host[%i] PING: Result %s\n", host->id, ping->ping_response));
+				SPINE_LOG_HIGH(("Host[%i] PING: Result %s", host->id, ping->ping_response));
 			}
 		}else{
 			if (availability_method == AVAIL_SNMP_AND_PING) {
-				SPINE_LOG_HIGH(("Host[%i] PING Result: %s\n", host->id, ping->ping_response));
-				SPINE_LOG_HIGH(("Host[%i] SNMP Result: %s\n", host->id, ping->snmp_response));
+				SPINE_LOG_HIGH(("Host[%i] PING Result: %s", host->id, ping->ping_response));
+				SPINE_LOG_HIGH(("Host[%i] SNMP Result: %s", host->id, ping->snmp_response));
 			}else if (availability_method == AVAIL_SNMP) {
-				SPINE_LOG_HIGH(("Host[%i] SNMP Result: %s\n", host->id, ping->snmp_response));
+				SPINE_LOG_HIGH(("Host[%i] SNMP Result: %s", host->id, ping->snmp_response));
 			}else if (availability_method == AVAIL_NONE) {
-				SPINE_LOG_HIGH(("Host[%i] No Host Availability Method Selected\n", host->id));
+				SPINE_LOG_HIGH(("Host[%i] No Host Availability Method Selected", host->id));
 			}else{
-				SPINE_LOG_HIGH(("Host[%i] PING Result: %s\n", host->id, ping->ping_response));
+				SPINE_LOG_HIGH(("Host[%i] PING Result: %s", host->id, ping->ping_response));
 			}
 		}
 	}
@@ -965,10 +1182,9 @@ void update_host_status(int status, host_t *host, ping_t *ping, int availability
 	/* if there is supposed to be an event generated, do it */
 	if (issue_log_message) {
 		if (host->status == HOST_DOWN) {
-			SPINE_LOG(("Host[%i] Hostname[%s] ERROR: HOST EVENT: Host is DOWN Message: %s\n", host->id, host->hostname, host->status_last_error));
+			SPINE_LOG(("Host[%i] Hostname[%s] ERROR: HOST EVENT: Host is DOWN Message: %s", host->id, host->hostname, host->status_last_error));
 		}else{
-			SPINE_LOG(("Host[%i] Hostnanme[%s] NOTICE: HOST EVENT: Host Returned from DOWN State\n", host->id, host->hostname));
+			SPINE_LOG(("Host[%i] Hostname[%s] NOTICE: HOST EVENT: Host Returned from DOWN State", host->id, host->hostname));
 		}
 	}
 }
-
