@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2002-2007 The Cacti Group                                 |
+ | Copyright (C) 2002-2013 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -32,6 +32,7 @@
 */
 
 extern void *child(void *arg);
-extern void poll_host(int host_id);
+extern void poll_host(int host_id, int host_thread, int last_host_thread, int host_data_ids, char* host_time);
 extern char *exec_poll(host_t *current_host, char *command);
+extern int is_multipart_output(char *result);
 extern int validate_result(char *result);
