@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2002-2007 The Cacti Group                                 |
+ | Copyright (C) 2002-2012 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -34,24 +34,6 @@
 #include "common.h"
 #include "spine.h"
 
-#ifdef USE_NET_SNMP
- #undef PACKAGE_NAME
- #undef PACKAGE_VERSION
- #undef PACKAGE_BUGREPORT
- #undef PACKAGE_STRING
- #undef PACKAGE_TARNAME
- #include <net-snmp/net-snmp-config.h>
- #include <net-snmp/utilities.h>
- #include <net-snmp/net-snmp-includes.h>
- #include <net-snmp/config_api.h>
- #include <net-snmp/mib_api.h>
-#else
- #include <ucd-snmp/ucd-snmp-config.h>
- #include <ucd-snmp/ucd-snmp-includes.h>
- #include <ucd-snmp/system.h>
- #include <mib.h>
-#endif
-
 /* resolve problems in debian */
 #ifndef NETSNMP_DS_LIB_DONT_PERSIST_STATE
  #define NETSNMP_DS_LIB_DONT_PERSIST_STATE 32
@@ -87,32 +69,33 @@ void snmp_spine_init(void) {
 	#endif
 
 	netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_QUICK_PRINT, 1);
+	netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_QUICKE_PRINT, 1);
 	netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_PRINT_BARE_VALUE, 1);
 	netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_NUMERIC_TIMETICKS, 1);
 
-	#ifdef PACKAGE_VERSION
+	#if defined(VERIFY_PACKAGE_VERSION) && defined(PACKAGE_VERSION)
 		/* check that the headers we compiled with match the library we linked with -
 		   apparently not defined in UCD-SNMP...
 		*/
-		SPINE_LOG_DEBUG(("DEBUG: SNMP Header Version is %s\n", PACKAGE_VERSION));
-		SPINE_LOG_DEBUG(("DEBUG: SNMP Library Version is %s\n", netsnmp_get_version()));
+		SPINE_LOG_DEBUG(("DEBUG: SNMP Header Version is %s", PACKAGE_VERSION));
+		SPINE_LOG_DEBUG(("DEBUG: SNMP Library Version is %s", netsnmp_get_version()));
 
 		if(STRMATCH(PACKAGE_VERSION,netsnmp_get_version())) {
-			init_snmp("snmpapp");
+			init_snmp("spine");
 		}else{
 			/* report the error and quit spine */
 			die("ERROR: SNMP Library Version Mismatch (%s vs %s)",PACKAGE_VERSION,netsnmp_get_version());
 		}
 	#else
-		SPINE_LOG_DEBUG(("DEBUG: Issues with SNMP Header Version information, assuming old version of Net-SNMP.\n"));
-		init_snmp("snmpapp");
+		SPINE_LOG_DEBUG(("DEBUG: Issues with SNMP Header Version information, assuming old version of Net-SNMP."));
+		init_snmp("spine");
 	#endif
 #else
 	ds_set_boolean(DS_LIBRARY_ID, DS_LIB_QUICK_PRINT, 1);
 	ds_set_boolean(DS_LIBRARY_ID, DS_LIB_PRINT_BARE_VALUE, 1);
 	ds_set_boolean(DS_LIBRARY_ID, DS_LIB_NUMERIC_TIMETICKS, 1);
 
-	init_snmp("snmpapp");
+	init_snmp("spine");
 #endif
 }
 
@@ -146,9 +129,16 @@ void *snmp_host_init(int host_id, char *hostname, int snmp_version, char *snmp_c
 	char   hostnameport[BUFSIZE];
 
 	/* initialize SNMP */
-  	snmp_sess_init(&session);
+	snmp_sess_init(&session);
 
 	#ifdef USE_NET_SNMP
+		/* Bind to snmp_clientaddr if specified */
+		if (NULL != set.snmp_clientaddr  && strlen(set.snmp_clientaddr) > 0) {
+			#if SNMP_LOCALNAME == 1
+			session.localname = strdup(set.snmp_clientaddr);
+			#endif
+		}
+
 		/* Prevent update of the snmpapp.conf file */
 		#ifdef NETSNMP_DS_LIB_DONT_PERSIST_STATE
 			netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_DONT_PERSIST_STATE, 1);
@@ -164,6 +154,7 @@ void *snmp_host_init(int host_id, char *hostname, int snmp_version, char *snmp_c
 		#endif
 
 		netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_QUICK_PRINT, 1);
+		netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_QUICKE_PRINT, 1);
 		netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_PRINT_BARE_VALUE, 1);
 		netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_NUMERIC_TIMETICKS, 1);
 	#else
@@ -172,15 +163,30 @@ void *snmp_host_init(int host_id, char *hostname, int snmp_version, char *snmp_c
 		ds_set_boolean(DS_LIBRARY_ID, DS_LIB_NUMERIC_TIMETICKS, 1);
 	#endif
 
+	session.securityEngineID = 0;
+	session.securityEngineIDLen = 0;
+
+	session.securityName = 0;
+	session.securityNameLen = 0;
+
+	session.contextEngineID = 0;
+	session.contextEngineIDLen = 0;
+
+	session.contextName = 0;
+	session.contextNameLen = 0;
+
 	/* verify snmp version is accurate */
 	if (snmp_version == 2) {
-		session.version = SNMP_VERSION_2c;
+		session.version       = SNMP_VERSION_2c;
+		session.securityModel = SNMP_SEC_MODEL_SNMPv2c;
 	}else if (snmp_version == 1) {
-		session.version = SNMP_VERSION_1;
+		session.version       = SNMP_VERSION_1;
+		session.securityModel = SNMP_SEC_MODEL_SNMPv1;
 	}else if (snmp_version == 3) {
-		session.version = SNMP_VERSION_3;
+		session.version       = SNMP_VERSION_3;
+		session.securityModel = USM_SEC_MODEL_NUMBER;
 	}else {
-		SPINE_LOG(("Host[%i] ERROR: SNMP Version Error for Host '%s'\n", host_id, hostname));
+		SPINE_LOG(("Host[%i] ERROR: SNMP Version Error for Host '%s'", host_id, hostname));
 		return 0;
 	}
 
@@ -191,27 +197,29 @@ void *snmp_host_init(int host_id, char *hostname, int snmp_version, char *snmp_c
 	session.timeout     = (snmp_timeout * 1000); /* net-snmp likes microseconds */
 
 	if ((snmp_version == 2) || (snmp_version == 1)) {
-		session.community     = snmp_community;
+		session.community     = (unsigned char*) snmp_community;
 		session.community_len = strlen(snmp_community);
 	}else {
-	    /* set the SNMPv3 user name */
-	    session.securityName         = snmp_username;
-	    session.securityNameLen      = strlen(session.securityName);
+		/* set the SNMPv3 user name */
+		session.securityName         = snmp_username;
+		session.securityNameLen      = strlen(session.securityName);
 
-		session.contextName          = snmp_context;
-		session.contextNameLen       = strlen(session.contextName);
+		if (snmp_context && strlen(snmp_context)) {
+			session.contextName          = snmp_context;
+			session.contextNameLen       = strlen(session.contextName);
+		}
 
 		session.securityAuthKeyLen   = USM_AUTH_KU_LEN;
 
 		/* set the authentication protocol */
 		if (strcmp(snmp_auth_protocol, "MD5") == 0) {
 			/* set the authentication method to MD5 */
-			session.securityAuthProto    = snmp_duplicate_objid(usmHMACMD5AuthProtocol, OIDSIZE(usmHMACMD5AuthProtocol));
-			session.securityAuthProtoLen = OIDSIZE(usmHMACMD5AuthProtocol);
+			session.securityAuthProto    = snmp_duplicate_objid(usmHMACMD5AuthProtocol, USM_AUTH_PROTO_MD5_LEN);
+			session.securityAuthProtoLen = USM_AUTH_PROTO_MD5_LEN;
 		}else{
 			/* set the authentication method to SHA1 */
-			session.securityAuthProto    = snmp_duplicate_objid(usmHMACSHA1AuthProtocol, OIDSIZE(usmHMACSHA1AuthProtocol));
-			session.securityAuthProtoLen = OIDSIZE(usmHMACSHA1AuthProtocol);
+			session.securityAuthProto    = snmp_duplicate_objid(usmHMACSHA1AuthProtocol, USM_AUTH_PROTO_SHA_LEN);
+			session.securityAuthProtoLen = USM_AUTH_PROTO_SHA_LEN;
 		}
 
 		/* set the authentication key to the hashed version. The password must me at least 8 char */
@@ -225,7 +233,7 @@ void *snmp_host_init(int host_id, char *hostname, int snmp_version, char *snmp_c
 		}
 
 		/* set the privacy protocol to none */
-		if (strcmp(snmp_priv_protocol, "[None]") == 0) {
+		if (strcmp(snmp_priv_protocol, "[None]") == 0 || (strlen(snmp_priv_passphrase) == 0)) {
 			session.securityPrivProto    = snmp_duplicate_objid(usmNoPrivProtocol, OIDSIZE(usmNoPrivProtocol));
 			session.securityPrivProtoLen = OIDSIZE(usmNoPrivProtocol);
 			session.securityPrivKeyLen   = USM_PRIV_KU_LEN;
@@ -234,16 +242,22 @@ void *snmp_host_init(int host_id, char *hostname, int snmp_version, char *snmp_c
 			session.securityLevel        = SNMP_SEC_LEVEL_AUTHNOPRIV;
 		}else{
 			if (strcmp(snmp_priv_protocol, "DES") == 0) {
-				session.securityPrivProto    = snmp_duplicate_objid(usmDESPrivProtocol, OIDSIZE(usmDESPrivProtocol));
-				session.securityPrivProtoLen = OIDSIZE(usmDESPrivProtocol);
+				session.securityPrivProto    = snmp_duplicate_objid(usmDESPrivProtocol, USM_PRIV_PROTO_DES_LEN);
+				session.securityPrivProtoLen = USM_PRIV_PROTO_DES_LEN;
 				session.securityPrivKeyLen   = USM_PRIV_KU_LEN;
 
 				/* set the security level to authenticate, and encrypted */
 				session.securityLevel        = SNMP_SEC_LEVEL_AUTHPRIV;
 			}else{
+				#if defined(USM_PRIV_PROTO_AES_LEN)
+				session.securityPrivProto    = snmp_duplicate_objid(usmAESPrivProtocol, USM_PRIV_PROTO_AES_LEN);
+				session.securityPrivProtoLen = USM_PRIV_PROTO_AES_LEN;
+				session.securityPrivKeyLen   = USM_PRIV_KU_LEN;
+				#else
 				session.securityPrivProto    = snmp_duplicate_objid(usmAES128PrivProtocol, OIDSIZE(usmAES128PrivProtocol));
 				session.securityPrivProtoLen = OIDSIZE(usmAES128PrivProtocol);
 				session.securityPrivKeyLen   = USM_PRIV_KU_LEN;
+				#endif
 
 				/* set the security level to authenticate, and encrypted */
 				session.securityLevel        = SNMP_SEC_LEVEL_AUTHPRIV;
@@ -262,12 +276,12 @@ void *snmp_host_init(int host_id, char *hostname, int snmp_version, char *snmp_c
 	}
 
 	/* open SNMP Session */
- 	thread_mutex_lock(LOCK_SNMP);
+	thread_mutex_lock(LOCK_SNMP);
 	sessp = snmp_sess_open(&session);
 	thread_mutex_unlock(LOCK_SNMP);
 
 	if (!sessp) {
-		SPINE_LOG(("ERROR: Problem initializing SNMP session '%s'\n", hostname));
+		SPINE_LOG_MEDIUM(("ERROR: Problem initializing SNMP session '%s'", hostname));
 	}
 
 	return sessp;
@@ -304,8 +318,9 @@ char *snmp_get(host_t *current_host, char *snmp_oid) {
 	oid    anOID[MAX_OID_LEN];
 	int    status;
 	char   *result_string;
+	char   temp_result[RESULTS_BUFFER];
 
-	if (!(result_string = (char *) malloc(BUFSIZE))) {
+	if (!(result_string = (char *) malloc(RESULTS_BUFFER))) {
 		die("ERROR: Fatal malloc error: snmp.c snmp_get!");
 	}
 	result_string[0] = '\0';
@@ -314,10 +329,10 @@ char *snmp_get(host_t *current_host, char *snmp_oid) {
 
 	if (current_host->snmp_session != NULL) {
 		anOID_len = MAX_OID_LEN;
-		pdu = snmp_pdu_create(SNMP_MSG_GET);
+		pdu       = snmp_pdu_create(SNMP_MSG_GET);
 
 		if (!snmp_parse_oid(snmp_oid, anOID, &anOID_len)) {
-			SPINE_LOG(("ERROR: Problems parsing SNMP OID\n"));
+			SPINE_LOG(("Host[%i] ERROR: SNMP Get Problems parsing SNMP OID %s", current_host->id, snmp_oid));
 			SET_UNDEFINED(result_string);
 			return result_string;
 		}else{
@@ -327,10 +342,13 @@ char *snmp_get(host_t *current_host, char *snmp_oid) {
 		/* poll host */
 		status = snmp_sess_synch_response(current_host->snmp_session, pdu, &response);
 
+		/* add status to host structure */
+		current_host->snmp_status = status;
+
 		/* liftoff, successful poll, process it!! */
 		if (status == STAT_SUCCESS) {
 			if (response == NULL) {
-				SPINE_LOG(("ERROR: An internal Net-Snmp error condition detected in Cacti snmp_get\n"));
+				SPINE_LOG(("ERROR: An internal Net-Snmp error condition detected in Cacti snmp_get"));
 
 				SET_UNDEFINED(result_string);
 				status = STAT_ERROR;
@@ -339,10 +357,12 @@ char *snmp_get(host_t *current_host, char *snmp_oid) {
 					vars = response->variables;
 
 					#ifdef USE_NET_SNMP
-					snprint_value(result_string, BUFSIZE, anOID, anOID_len, vars);
+					snmp_snprint_value(temp_result, RESULTS_BUFFER, vars->name, vars->name_length, vars);
 					#else
-					sprint_value(result_string, anOID, anOID_len, vars);
+					sprint_value(temp_result, anOID, anOID_len, vars);
 					#endif
+
+					snprintf(result_string, RESULTS_BUFFER, "%s", trim(temp_result));
 				}
 			}
 		}
@@ -382,20 +402,21 @@ char *snmp_getnext(host_t *current_host, char *snmp_oid) {
 	oid    anOID[MAX_OID_LEN];
 	int    status;
 	char   *result_string;
+	char   temp_result[RESULTS_BUFFER];
 
-	if (!(result_string = (char *) malloc(BUFSIZE))) {
+	if (!(result_string = (char *) malloc(RESULTS_BUFFER))) {
 		die("ERROR: Fatal malloc error: snmp.c snmp_get!");
 	}
 	result_string[0] = '\0';
 
-	status           = STAT_DESCRIP_ERROR;
+	status = STAT_DESCRIP_ERROR;
 
 	if (current_host->snmp_session != NULL) {
 		anOID_len = MAX_OID_LEN;
 		pdu       = snmp_pdu_create(SNMP_MSG_GETNEXT);
 
 		if (!snmp_parse_oid(snmp_oid, anOID, &anOID_len)) {
-			SPINE_LOG(("ERROR: Problems parsing SNMP OID\n"));
+			SPINE_LOG(("Host[%i] ERROR: SNMP Getnext Problems parsing SNMP OID %s", current_host->id, snmp_oid));
 			SET_UNDEFINED(result_string);
 			return result_string;
 		}else{
@@ -405,10 +426,13 @@ char *snmp_getnext(host_t *current_host, char *snmp_oid) {
 		/* poll host */
 		status = snmp_sess_synch_response(current_host->snmp_session, pdu, &response);
 
+		/* add status to host structure */
+		current_host->snmp_status = status;
+
 		/* liftoff, successful poll, process it!! */
 		if (status == STAT_SUCCESS) {
 			if (response == NULL) {
-				SPINE_LOG(("ERROR: An internal Net-Snmp error condition detected in Cacti snmp_get\n"));
+				SPINE_LOG(("ERROR: An internal Net-Snmp error condition detected in Cacti snmp_get"));
 
 				SET_UNDEFINED(result_string);
 				status = STAT_ERROR;
@@ -416,11 +440,18 @@ char *snmp_getnext(host_t *current_host, char *snmp_oid) {
 				if (response->errstat == SNMP_ERR_NOERROR) {
 					vars = response->variables;
 
-					#ifdef USE_NET_SNMP
-					snprint_value(result_string, BUFSIZE, anOID, anOID_len, vars);
-					#else
-					sprint_value(result_string, anOID, anOID_len, vars);
-					#endif
+					if (vars != NULL) {
+						#ifdef USE_NET_SNMP
+						snmp_snprint_value(temp_result, RESULTS_BUFFER, vars->name, vars->name_length, vars);
+						#else
+						sprint_value(temp_result, anOID, anOID_len, vars);
+						#endif
+
+						snprintf(result_string, RESULTS_BUFFER, "%s", trim(temp_result));
+					}else{
+						SET_UNDEFINED(result_string);
+						status = STAT_ERROR;
+					}
 				}
 			}
 		}
@@ -442,6 +473,113 @@ char *snmp_getnext(host_t *current_host, char *snmp_oid) {
 	return result_string;
 }
 
+/*! \fn char *snmp_count(host_t *current_host, char *snmp_oid)
+ *  \brief counts entries of snmp table specified by a specific snmp OID
+ *
+ *	This function will poll a specific snmp OID for a host.  The host snmp
+ *  session must already be established.
+ *
+ *  \return returns count of table entries
+ *
+ */
+int snmp_count(host_t *current_host, char *snmp_oid) {
+	struct snmp_pdu *pdu       = NULL;
+	struct snmp_pdu *response  = NULL;
+	struct variable_list *vars = NULL;
+	size_t anOID_len           = MAX_OID_LEN;
+	size_t rootlen             = MAX_OID_LEN;
+	oid    anOID[MAX_OID_LEN];
+	oid    root[MAX_OID_LEN];
+	int    status;
+	int    ok = 1;
+	int    error_occurred = 0;
+	int    count = 0;
+	char   temp_result[RESULTS_BUFFER];
+
+	status = STAT_DESCRIP_ERROR;
+
+	SPINE_LOG_DEBUG(("NOTE: walk starts at OID %s", snmp_oid));
+
+	if (current_host->snmp_session != NULL) {
+		rootlen = MAX_OID_LEN;
+		/* parse input parm to an array for use with snmp functions */
+		if (!snmp_parse_oid(snmp_oid, root, &rootlen)) {
+			SPINE_LOG(("Host[%i] ERROR: SNMP Count Problems parsing SNMP OID %s", current_host->id, snmp_oid));
+			return count;
+		}
+		memmove(anOID, root, rootlen * sizeof(oid));
+		anOID_len = rootlen;
+
+		while (ok && !error_occurred) {
+			/* create PDU for GETNEXT request */
+			pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
+			snmp_add_null_var(pdu, anOID, anOID_len);
+
+			/* do the request, use thread safe call */
+			status = snmp_sess_synch_response(current_host->snmp_session, pdu, &response);
+
+			/* add status to host structure */
+			current_host->snmp_status = status;
+
+			//SPINE_LOG_DEBUG(("TRACE: Status %i Response %i", status, response->errstat));
+
+			if (status == STAT_SUCCESS) {
+				if (response->errstat == SNMP_ERR_NOERROR) {
+					/* check resulting variables */
+					for (vars = response->variables; vars; vars	= vars->next_variable) {
+						if ((vars->name_length < rootlen) || (memcmp(root, vars->name, rootlen * sizeof(oid)) != 0)) {
+							/* next OID is not part of snmptable */
+							ok = 0;
+							continue;
+						}
+						count++;
+
+						/* END OF MIB or NO SUCH OBJECT or NO SUCH INSTANCE */
+						if ((vars->type != SNMP_ENDOFMIBVIEW) &&
+							(vars->type	!= SNMP_NOSUCHOBJECT) &&
+							(vars->type	!= SNMP_NOSUCHINSTANCE)) {
+							/* valid data, so perform a compare  */
+							if (snmp_oid_compare(anOID, anOID_len, vars->name, vars->name_length) >= 0) {
+								SPINE_LOG(("ERROR: OID not increasing"));
+								ok = 0;
+								error_occurred = 1;
+							}
+							/* prepare next turn */
+							memmove((char *) anOID, (char *) vars->name, vars->name_length * sizeof(oid));
+							anOID_len = vars->name_length;
+						} else {
+							/* abnormal end of loop */
+							ok = 0;
+						}
+					}
+				} else {
+					SPINE_LOG(("ERROR: An internal Net-Snmp error condition detected in Cacti snmp_count"));
+				}
+			} else if (status == STAT_TIMEOUT) {
+				SPINE_LOG(("ERROR: Timeout detected in Cacti snmp_count"));
+				ok = 0;
+				error_occurred = 1;
+			} else { /* status == STAT_ERROR */
+				SPINE_LOG(("ERROR: An internal Net-Snmp error condition detected in Cacti snmp_count (STAT_ERROR)"));
+				ok = 0;
+				error_occurred = 1;
+			}
+
+			if (response) {
+				snmp_free_pdu(response);
+			}
+		}
+	}else{
+		status = STAT_DESCRIP_ERROR;
+	}
+
+	if (status != STAT_SUCCESS) {
+		current_host->ignore_host = TRUE;
+	}
+
+	return count;
+}
+
 /*! \fn void snmp_snprint_value(char *obuf, size_t buf_len, const oid *objid, size_t objidlen, struct variable_list *variable)
  *
  *  \brief replacement for the buggy net-snmp.org snprint_value function
@@ -455,18 +593,18 @@ void snmp_snprint_value(char *obuf, size_t buf_len, const oid *objid, size_t obj
 	u_char *buf    = NULL;
 	size_t out_len = 0;
 
-	if ((buf = (u_char *) calloc(buf_len, 1)) != 0) {
-		if (sprint_realloc_value(&buf, &buf_len, &out_len, 1,
-				objid, objidlen, variable)) {
+	if (buf_len > 0) {
+		if ((buf = (u_char *) calloc(buf_len, 1)) != 0) {
+			sprint_realloc_by_type(&buf, &buf_len, &out_len, 1, variable, NULL, NULL, NULL);
 			snprintf(obuf, buf_len, "%s", buf);
 		}else{
-			snprintf(obuf, buf_len, "%s [TRUNCATED]", buf);
+			SET_UNDEFINED(obuf);
 		}
+
+		free(buf);
 	}else{
 		SET_UNDEFINED(obuf);
 	}
-
-	free(buf);
 }
 
 /*! \fn char *snmp_get_multi(host_t *current_host, snmp_oids_t *snmp_oids, int num_oids)
@@ -483,14 +621,13 @@ void snmp_get_multi(host_t *current_host, snmp_oids_t *snmp_oids, int num_oids)
 	struct variable_list *vars = NULL;
 	int status;
 	int i;
-	int max_repetitions = 1;
-	int non_repeaters   = 0;
 	int array_count;
 	int index_count;
+	char   temp_result[RESULTS_BUFFER];
 
 	struct nameStruct {
-	    oid             name[MAX_OID_LEN];
-	    size_t          name_len;
+		oid             name[MAX_OID_LEN];
+		size_t          name_len;
 	} *name, *namep;
 
 	/* load up oids */
@@ -500,9 +637,9 @@ void snmp_get_multi(host_t *current_host, snmp_oids_t *snmp_oids, int num_oids)
 		namep->name_len = MAX_OID_LEN;
 
 		if (!snmp_parse_oid(snmp_oids[i].oid, namep->name, &namep->name_len)) {
- 			SPINE_LOG(("Host[%i] ERROR: Problems parsing Multi SNMP OID! (oid: %s)\n", current_host->id, snmp_oids[i].oid));
+			SPINE_LOG(("Host[%i] ERROR: Problems parsing Multi SNMP OID! (oid: %s), Set MAX_OIDS to 1 for this host to isolate bad OID", current_host->id, snmp_oids[i].oid));
 
- 			/* Mark this OID as "bad" */
+			/* Mark this OID as "bad" */
 			SET_UNDEFINED(snmp_oids[i].result);
 		}else{
 			snmp_add_null_var(pdu, namep->name, namep->name_len);
@@ -517,10 +654,13 @@ void snmp_get_multi(host_t *current_host, snmp_oids_t *snmp_oids, int num_oids)
 	retry:
 	status = snmp_sess_synch_response(current_host->snmp_session, pdu, &response);
 
+	/* add status to host structure */
+	current_host->snmp_status = status;
+
 	/* liftoff, successful poll, process it!! */
 	if (status == STAT_SUCCESS) {
 		if (response == NULL) {
-			SPINE_LOG(("ERROR: An internal Net-Snmp error condition detected in Cacti snmp_get_multi\n"));
+			SPINE_LOG(("ERROR: An internal Net-Snmp error condition detected in Cacti snmp_get_multi"));
 			status = STAT_ERROR;
 		}else{
 			if (response->errstat == SNMP_ERR_NOERROR) {
@@ -529,10 +669,12 @@ void snmp_get_multi(host_t *current_host, snmp_oids_t *snmp_oids, int num_oids)
 				for(i = 0; i < num_oids && vars; i++) {
 					if (!IS_UNDEFINED(snmp_oids[i].result)) {
 						#ifdef USE_NET_SNMP
-						snmp_snprint_value(snmp_oids[i].result, sizeof(snmp_oids[i].result), vars->name, vars->name_length, vars);
+						snmp_snprint_value(temp_result, RESULTS_BUFFER, vars->name, vars->name_length, vars);
 						#else
-						sprint_value(snmp_oids[i].result, vars->name, vars->name_length, vars);
+						sprint_value(temp_result, vars->name, vars->name_length, vars);
 						#endif
+
+						snprintf(snmp_oids[i].result, RESULTS_BUFFER, "%s", trim(temp_result));
 						
 						vars = vars->next_variable;
 					}
@@ -559,7 +701,7 @@ void snmp_get_multi(host_t *current_host, snmp_oids_t *snmp_oids, int num_oids)
 
 					}
 
-					/* remote the invalid OID from the PDU */
+					/* remove the invalid OID from the PDU */
 					pdu = snmp_fix_pdu(response, SNMP_MSG_GET);
 
 					/* free the previous response */
@@ -570,7 +712,7 @@ void snmp_get_multi(host_t *current_host, snmp_oids_t *snmp_oids, int num_oids)
 						/* retry the request */
 						goto retry;
 					}else{
-					    /* all OID's errored out so exit cleanly */
+						/* all OID's errored out so exit cleanly */
 						status = STAT_SUCCESS;
 					}
 				}else{
