@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2002-2007 The Cacti Group                                 |
+ | Copyright (C) 2002-2013 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -40,7 +40,7 @@
  * -C | --conf=F
  *
  *	Provide the name of the Spine configuration file, which contains
- *	the parameters for connecting to the database. In the absense of
+ *	the parameters for connecting to the database. In the absence of
  *	this, it looks [WHERE?]
  *
  * -f | --first=ID
@@ -51,6 +51,10 @@
  *
  *	Stop polling after device <ID> (else ends with the last one)
  *
+ * -H | --hostlist="hostid1,hostid2,hostid3,...,hostidn"
+ *
+ *	Override the expected first host, last host behavior with a list of hostids.
+ *
  * -O | --option=setting:value
  *
  *	Override a DB-provided value from the settings table in the DB.
@@ -90,6 +94,7 @@ int entries = 0;
 int num_hosts = 0;
 int active_threads = 0;
 int active_scripts = 0;
+int thread_ready   = FALSE;
 
 config_t set;
 php_t	*php_processes = 0;
@@ -98,6 +103,55 @@ char	 config_paths[CONFIG_PATHS][BUFSIZE];
 static char *getarg(char *opt, char ***pargv);
 static void display_help(void);
 
+#ifdef HAVE_LCAP
+/* This patch is adapted (copied) patch for ntpd from Jarno Huuskonen and
+ * Pekka Savola that was adapted (copied) from a patch by Chris Wings to drop
+ * root for xntpd.
+ */
+void drop_root(uid_t server_uid, gid_t server_gid) {
+	cap_t caps;
+	if (prctl(PR_SET_KEEPCAPS, 1)) {
+		SPINE_LOG_HIGH(("prctl(PR_SET_KEEPCAPS, 1) failed"));
+		exit(1);
+	}
+
+	if ( setgroups(0, NULL) == -1 ) {
+		SPINE_LOG_HIGH(("setgroups failed."));
+		exit(1);
+	}
+
+	if ( setegid(server_gid) == -1 || seteuid(server_uid) == -1 ) {
+		SPINE_LOG_HIGH(("setegid/seteuid to uid=%d/gid=%d failed.", server_uid,
+			server_gid));
+		exit(1);
+	}
+
+	caps = cap_from_text("cap_net_raw=eip");
+	if (caps == NULL) {
+		SPINE_LOG_HIGH(("cap_from_text failed."));
+		exit(1);
+	}
+
+	if (cap_set_proc(caps) == -1) {
+		SPINE_LOG_HIGH(("cap_set_proc failed."));
+		exit(1);
+	}
+
+	/* Try to free the memory from cap_from_text */
+	cap_free( caps );
+
+	if ( setregid(server_gid, server_gid) == -1 ||
+		setreuid(server_uid, server_uid) == -1 ) {
+		SPINE_LOG_HIGH(("setregid/setreuid to uid=%d/gid=%d failed.",
+			server_uid, server_gid));
+		exit(1);
+	}
+
+	SPINE_LOG_LOW(("running as uid(%d)/gid(%d) euid(%d)/egid(%d) with cap_net_raw=eip.",
+		getuid(), getgid(), geteuid(), getegid()));
+}
+#endif /* HAVE_LCAP */
+
 /*! \fn main(int argc, char *argv[])
  *  \brief The Spine program entry point
  *  \param argc The number of arguments passed to the function plus one (+1)
@@ -125,27 +179,39 @@ int main(int argc, char *argv[]) {
 	struct timeval now;
 	char *conf_file = NULL;
 	double begin_time, end_time, current_time;
-	int poller_interval;
-	int num_rows;
+	int num_rows = 0;
 	int device_counter = 0;
 	int poller_counter = 0;
 	int last_active_threads = 0;
 	int valid_conf_file = FALSE;
-	long int EXTERNAL_THREAD_SLEEP = 5000;
+	long int EXTERNAL_THREAD_SLEEP = 50;
 	long int internal_thread_sleep;
+	char querybuf[BIG_BUFSIZE], *qp = querybuf;
+	char *host_time = NULL;
+	int itemsPT = 0;
+	int device_threads;
+
+	#ifdef HAVE_LCAP
+	if (geteuid() == 0)
+		drop_root(getuid(), getgid());
+	#endif /* HAVE_LCAP */
 
 	pthread_t* threads = NULL;
+	poller_thread_t* poller_details = NULL;
 	pthread_attr_t attr;
 
 	int* ids = NULL;
 	MYSQL mysql;
-	MYSQL_RES *result = NULL;
+	MYSQL_RES *result  = NULL;
+	MYSQL_RES *tresult = NULL;
 	MYSQL_ROW mysql_row;
-	int canexit = 0;
-	int host_id;
+	int canexit = FALSE;
+	int host_id = 0;
 	int i;
-	int mutex_status = 0;
+	int mutex_status  = 0;
 	int thread_status = 0;
+	int change_host   = TRUE;
+	int current_thread;
 
 	UNUSED_PARAMETER(argc);		/* we operate strictly with argv */
 
@@ -158,6 +224,9 @@ int main(int argc, char *argv[]) {
 		php_processes[i].php_state = PHP_BUSY;
 	}
 
+	/* initialize icmp_avail */
+	set.icmp_avail = TRUE;
+
 	/* detect and compensate for stdin/stderr ttys */
 	if (!isatty(fileno(stdout))) {
 		set.stdout_notty = TRUE;
@@ -209,8 +278,10 @@ int main(int argc, char *argv[]) {
 	 */
 
 	/* initialize some global variables */
+	set.poller_id         = 0;
 	set.start_host_id     = -1;
 	set.end_host_id       = -1;
+	set.host_id_list[0]   = '\0';
 	set.php_initialized   = FALSE;
 	set.logfile_processed = FALSE;
 	set.parent_fork       = SPINE_PARENT;
@@ -252,6 +323,11 @@ int main(int argc, char *argv[]) {
 			set.poller_id = atoi(getarg(opt, &argv));
 		}
 
+		else if (STRMATCH(arg, "-H") ||
+				 STRIMATCH(arg, "--hostlist")) {
+			snprintf(set.host_id_list, BIG_BUFSIZE, "%s", getarg(opt, &argv));
+		}
+
 		else if (STRMATCH(arg, "-h") ||
 				 STRMATCH(arg, "-v") ||
 				 STRMATCH(arg, "--help") ||
@@ -319,9 +395,26 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
+	/* we attempt to support scripts better in cygwin */
+	#if defined(__CYGWIN__)
+	setenv("CYGWIN", "nodosfilewarning", 1);
+	if (file_exists("./sh.exe")) {
+		set.cygwinshloc = 0;
+		if (set.log_level == POLLER_VERBOSITY_DEBUG) {
+			printf("NOTE: The Shell Command Exists in the current directory\n");
+		}
+	}else{
+		set.cygwinshloc = 1;
+		if (set.log_level == POLLER_VERBOSITY_DEBUG) {
+			printf("NOTE: The Shell Command Exists in the /bin directory\n");
+		}
+	}
+	#endif
+
 	/* we require either both the first and last hosts, or niether host */
-	if (HOSTID_DEFINED(set.start_host_id) != HOSTID_DEFINED(set.end_host_id)) {
-		die("ERROR: must provide both -f/-l, or neither");
+	if ((HOSTID_DEFINED(set.start_host_id) != HOSTID_DEFINED(set.end_host_id)) &&
+		(!strlen(set.host_id_list))) {
+		die("ERROR: must provide both -f/-l, a hostlist (-H/--hostlist), or neither");
 	}
 
 	if (set.start_host_id > set.end_host_id) {
@@ -363,44 +456,44 @@ int main(int argc, char *argv[]) {
 
 	/* set the poller interval for those who use less than 5 minute intervals */
 	if (set.poller_interval == 0) {
-		poller_interval = 300;
-	}else {
-		poller_interval = set.poller_interval;
+		set.poller_interval = 300;
 	}
 
 	/* calculate the external_tread_sleep value */
 	internal_thread_sleep = EXTERNAL_THREAD_SLEEP * set.num_parent_processes / 50;
 
+	/* connect to database */
+	db_connect(set.dbdb, &mysql);
+
 	if (set.log_level == POLLER_VERBOSITY_DEBUG) {
-		SPINE_LOG_DEBUG(("Version %s starting\n", VERSION));
+		SPINE_LOG_DEBUG(("Version %s starting", VERSION));
 	}else{
 		if (!set.stdout_notty) {
 			printf("SPINE: Version %s starting\n", VERSION);
 		}
 	}
 
-	/* connect to database */
-	db_connect(set.dbdb, &mysql);
-
 	/* see if mysql is thread safe */
 	if (mysql_thread_safe()) {
 		if (set.log_level == POLLER_VERBOSITY_DEBUG) {
 			SPINE_LOG(("DEBUG: MySQL is Thread Safe!"));
 		}
 	}else{
-		SPINE_LOG(("WARNING: MySQL is NOT Thread Safe!\n"));
+		SPINE_LOG(("WARNING: MySQL is NOT Thread Safe!"));
 	}
 
+	/* test for asroot permissions for ICMP */
+	checkAsRoot();
+
 	/* initialize SNMP */
-	SPINE_LOG_DEBUG(("SPINE: Initializing Net-SNMP API\n"));
+	SPINE_LOG_DEBUG(("SPINE: Initializing Net-SNMP API"));
 	snmp_spine_init();
 
 	/* initialize PHP if required */
-	SPINE_LOG_DEBUG(("SPINE: Initializing PHP Script Server(s)\n"));
+	SPINE_LOG_DEBUG(("SPINE: Initializing PHP Script Server(s)"));
 
 	/* tell spine that it is parent, and set the poller id */
 	set.parent_fork = SPINE_PARENT;
-	set.poller_id = 0;
 
 	/* initialize the script server */
 	if (set.php_required) {
@@ -409,19 +502,56 @@ int main(int argc, char *argv[]) {
 		set.php_current_server = 0;
 	}
 
-	/* obtain the list of hosts to poll */
-	{
-	char querybuf[SMALL_BUFSIZE], *qp = querybuf;
+	/* determine if the poller_id field exists in the host table */
+	result = db_query(&mysql, "SHOW COLUMNS FROM host LIKE 'poller_id'");
+	if (mysql_num_rows(result)) {
+		set.poller_id_exists = TRUE;
+	}else{
+		set.poller_id_exists = FALSE;
 
-	qp += sprintf(qp, "SELECT id FROM host");
+		if (set.poller_id > 0) {
+			SPINE_LOG(("WARNING: PollerID > 0, but 'host' table does NOT contain the poller_id column!!"));
+		}
+	}
+
+	/* determine if the device_threads field exists in the host table */
+	result = db_query(&mysql, "SHOW COLUMNS FROM host LIKE 'device_threads'");
+	if (mysql_num_rows(result)) {
+		set.device_threads_exists = TRUE;
+	}else{
+		set.device_threads_exists = FALSE;
+	}
+
+	if (set.device_threads_exists) {
+		SPINE_LOG_MEDIUM(("NOTE: Spine will support multithread device polling."));
+	}else{
+		SPINE_LOG_MEDIUM(("NOTE: Spine did not detect multithreaded device polling."));  
+	}
+
+	/* obtain the list of hosts to poll */
+	if (set.device_threads_exists) {
+		qp += sprintf(qp, "SELECT id, device_threads FROM host");
+	}else{
+		qp += sprintf(qp, "SELECT id, '1' as device_threads FROM host");
+	}
 	qp += sprintf(qp, " WHERE disabled=''");
-	qp += append_hostrange(qp, "id");	/* AND id BETWEEN a AND b */
+	if (!strlen(set.host_id_list)) {
+		qp += append_hostrange(qp, "id");	/* AND id BETWEEN a AND b */
+	}else{
+		qp += sprintf(qp, " AND id IN(%s)", set.host_id_list);
+	}
+	if (set.poller_id_exists) {
+		qp += sprintf(qp, " AND host.poller_id=%i", set.poller_id);
+	}
 	qp += sprintf(qp, " ORDER BY id");
 
 	result = db_query(&mysql, querybuf);
-	}
 
-	num_rows = mysql_num_rows(result) + 1; /* add 1 for host = 0 */
+	if (set.poller_id == 0) {
+		num_rows = mysql_num_rows(result) + 1; /* add 1 for host = 0 */
+	}else{
+		num_rows = mysql_num_rows(result); /* pollerid 0 takes care of not host based data sources */
+	}
 
 	if (!(threads = (pthread_t *)malloc(num_rows * sizeof(pthread_t)))) {
 		die("ERROR: Fatal malloc error: spine.c threads!");
@@ -431,6 +561,15 @@ int main(int argc, char *argv[]) {
 		die("ERROR: Fatal malloc error: spine.c host id's!");
 	}
 
+	/* mark the spine process as started */
+	if (!set.pre087g) {
+		SPINE_LOG_MEDIUM(("NOTE: Spine is behaving in a 0.8.7g+ manner"));
+		snprintf(querybuf, BIG_BUFSIZE, "INSERT INTO poller_time (poller_id, pid, start_time, end_time) VALUES (%i, %i, NOW(), '0000-00-00 00:00:00')", set.poller_id, getpid());
+		db_insert(&mysql, querybuf);
+	}else{
+		SPINE_LOG_MEDIUM(("NOTE: Spine is behaving in a PRE 0.8.7g manner"));
+	}
+
 	/* initialize threads and mutexes */
 	pthread_attr_init(&attr);
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
@@ -442,60 +581,117 @@ int main(int argc, char *argv[]) {
 	/* tell fork processes that they are now active */
 	set.parent_fork = SPINE_FORK;
 
+	/* initialize the threading code */
+	device_threads   = 1;
+	current_thread   = 0;
+
+	/* poller 0 always polls host 0 */
+	if (set.poller_id == 0) {
+		host_id     = 0;
+		change_host = FALSE;
+	}else{
+		change_host = TRUE;
+	}
+
 	/* loop through devices until done */
-	while ((device_counter < num_rows) && (canexit == 0)) {
-		mutex_status = thread_mutex_trylock(LOCK_THREAD);
-
-		switch (mutex_status) {
-		case 0:
-			if (last_active_threads != active_threads) {
+	while ((device_counter < num_rows) && (canexit == FALSE)) {
+		while ((active_threads < set.threads) && (device_counter < num_rows) && (canexit == FALSE)) {
+			mutex_status = thread_mutex_trylock(LOCK_THREAD);
+	
+			switch (mutex_status) {
+			case 0:
 				last_active_threads = active_threads;
-			}
-
-			while ((active_threads < set.threads) && (device_counter < num_rows)) {
-				if (device_counter > 0) {
-					mysql_row = mysql_fetch_row(result);
-					host_id = atoi(mysql_row[0]);
-					ids[device_counter] = host_id;
+	
+				if (change_host) {
+					mysql_row       = mysql_fetch_row(result);
+					host_id         = atoi(mysql_row[0]);
+					device_threads  = atoi(mysql_row[1]);
+					current_thread  = 1;
 				}else{
-					ids[device_counter] = 0;
+					current_thread++;
 				}
 
+				if (current_thread >= device_threads) {
+					change_host = TRUE;
+				}else{
+					change_host = FALSE;
+				}
+
+				/* determine how many items will be polled per thread */
+				if (device_threads > 1) {
+					if (current_thread == 1) {
+						snprintf(querybuf, BIG_BUFSIZE, "SELECT CEIL(COUNT(*)/%i) FROM poller_item WHERE host_id=%i", device_threads, host_id);
+						tresult   = db_query(&mysql, querybuf);
+						mysql_row = mysql_fetch_row(tresult);
+						itemsPT   = atoi(mysql_row[0]);
+						if (host_time) free(host_time);
+						host_time = get_host_poll_time();
+					}
+				}else{
+					itemsPT   = 0;
+					if (host_time) free(host_time);
+					host_time = get_host_poll_time();
+				}
+
+				/* populate the thread structure */
+				if (!(poller_details = (poller_thread_t *)malloc(sizeof(poller_thread_t)))) {
+					die("ERROR: Fatal malloc error: spine.c poller_details!");
+				}
+
+				poller_details->host_id          = host_id;
+				poller_details->host_thread      = current_thread;
+				poller_details->last_host_thread = device_threads;
+				poller_details->host_data_ids    = itemsPT;
+				poller_details->host_time        = host_time;
+
+				/* this variable tells us that the child had loaded the poller
+				 * poller_details structure and we can move on to the next thread
+				 */
+				thread_ready = FALSE;
+
 				/* create child process */
-				thread_status = pthread_create(&threads[device_counter], &attr, child, &ids[device_counter]);
+				thread_status = pthread_create(&threads[device_counter], &attr, child, poller_details);
 
 				switch (thread_status) {
 					case 0:
-						SPINE_LOG_DEBUG(("DEBUG: Valid Thread to be Created\n"));
+						SPINE_LOG_DEBUG(("DEBUG: Valid Thread to be Created"));
 
-						device_counter++;
+						if (change_host) {
+							device_counter++;
+						}
 						active_threads++;
 
-						SPINE_LOG_DEBUG(("DEBUG: The Value of Active Threads is %i\n", active_threads));
+						/* wait for the child to read and process the structure */
+						while (!thread_ready) { 
+							usleep(internal_thread_sleep);
+						}
+
+						SPINE_LOG_DEBUG(("DEBUG: The Value of Active Threads is %i", active_threads));
 
 						break;
 					case EAGAIN:
-						SPINE_LOG(("ERROR: The System Lacked the Resources to Create a Thread\n"));
+						SPINE_LOG(("ERROR: The System Lacked the Resources to Create a Thread"));
 						break;
 					case EFAULT:
-						SPINE_LOG(("ERROR: The Thread or Attribute Was Invalid\n"));
+						SPINE_LOG(("ERROR: The Thread or Attribute were Invalid"));
 						break;
 					case EINVAL:
-						SPINE_LOG(("ERROR: The Thread Attribute is Not Initialized\n"));
+						SPINE_LOG(("ERROR: The Thread Attribute is Not Initialized"));
 						break;
 					default:
-						SPINE_LOG(("ERROR: Unknown Thread Creation Error\n"));
+						SPINE_LOG(("ERROR: Unknown Thread Creation Error"));
 						break;
 				}
-				usleep(internal_thread_sleep);
+
+				thread_mutex_unlock(LOCK_THREAD);
 
 				/* get current time and exit program if time limit exceeded */
 				if (poller_counter >= 20) {
 					current_time = get_time_as_double();
 
-					if ((current_time - begin_time + 6) > poller_interval) {
-						SPINE_LOG(("ERROR: Spine Timed Out While Processing Hosts Internal\n"));
-						canexit = 1;
+					if ((current_time - begin_time + .2) > set.poller_interval) {
+						SPINE_LOG(("ERROR: Spine Timed Out While Processing Hosts Internal"));
+						canexit = TRUE;
 						break;
 					}
 
@@ -503,54 +699,50 @@ int main(int argc, char *argv[]) {
 				}else{
 					poller_counter++;
 				}
+	
+				break;
+			case EDEADLK:
+				SPINE_LOG(("ERROR: Deadlock Occured"));
+				break;
+			case EBUSY:
+				break;
+			case EINVAL:
+				SPINE_LOG(("ERROR: Attempt to Unlock an Uninitialized Mutex"));
+				break;
+			case EFAULT:
+				SPINE_LOG(("ERROR: Attempt to Unlock an Invalid Mutex"));
+				break;
+			default:
+				SPINE_LOG(("ERROR: Unknown Mutex Lock Error Code Returned"));
+				break;
+			}
+	
+			/* get current time and exit program if time limit exceeded */
+			if (poller_counter >= 20) {
+				current_time = get_time_as_double();
+	
+				if ((current_time - begin_time + .2) > set.poller_interval) {
+					SPINE_LOG(("ERROR: Spine Timed Out While Processing Hosts Internal"));
+					canexit = TRUE;
+					break;
+				}
+	
+				poller_counter = 0;
+			}else{
+				poller_counter++;
 			}
-
-			thread_mutex_unlock(LOCK_THREAD);
-
-			break;
-		case EDEADLK:
-			SPINE_LOG(("ERROR: Deadlock Occured\n"));
-			break;
-		case EBUSY:
-			break;
-		case EINVAL:
-			SPINE_LOG(("ERROR: Attempt to Unlock an Uninitialized Mutex\n"));
-			break;
-		case EFAULT:
-			SPINE_LOG(("ERROR: Attempt to Unlock an Invalid Mutex\n"));
-			break;
-		default:
-			SPINE_LOG(("ERROR: Unknown Mutex Lock Error Code Returned\n"));
-			break;
 		}
 
 		usleep(internal_thread_sleep);
-
-		/* get current time and exit program if time limit exceeded */
-		if (poller_counter >= 20) {
-			current_time = get_time_as_double();
-
-			if ((current_time - begin_time + 6) > poller_interval) {
-				SPINE_LOG(("ERROR: Spine Timed Out While Processing Hosts Internal\n"));
-				canexit = 1;
-				break;
-			}
-
-			poller_counter = 0;
-		}else{
-			poller_counter++;
-		}
 	}
 
 	/* wait for all threads to complete */
-	while (canexit == 0) {
+	while (canexit == FALSE) {
 		if (thread_mutex_trylock(LOCK_THREAD) == 0) {
-			if (last_active_threads != active_threads) {
-				last_active_threads = active_threads;
-			}
+			last_active_threads = active_threads;
 
 			if (active_threads == 0) {
-				canexit = 1;
+				canexit = TRUE;
 			}
 
 			thread_mutex_unlock(LOCK_THREAD);
@@ -562,9 +754,9 @@ int main(int argc, char *argv[]) {
 		if (poller_counter >= 20) {
 			current_time = get_time_as_double();
 
-			if ((current_time - begin_time + 6) > poller_interval) {
-				SPINE_LOG(("ERROR: Spine Timed Out While Processing Hosts Internal\n"));
-				canexit = 1;
+			if ((current_time - begin_time + .2) > set.poller_interval) {
+				SPINE_LOG(("ERROR: Spine Timed Out While Processing Hosts Internal"));
+				canexit = TRUE;
 				break;
 			}
 
@@ -582,43 +774,50 @@ int main(int argc, char *argv[]) {
 
 	/* update the db for |data_time| on graphs */
 	db_insert(&mysql, "replace into settings (name,value) values ('date',NOW())");
-	db_insert(&mysql, "insert into poller_time (poller_id, start_time, end_time) values (0, NOW(), NOW())");
+
+	/* setup poller_time depending on Cacti version */
+	if (set.pre087g) {
+		snprintf(querybuf, BIG_BUFSIZE, "INSERT INTO poller_time (poller_id, pid, start_time, end_time) VALUES (%i, %i, NOW(), NOW())", set.poller_id, getpid());
+	}else{
+		snprintf(querybuf, BIG_BUFSIZE, "UPDATE poller_time SET end_time=NOW() WHERE poller_id=%i AND pid=%i", set.poller_id, getpid());
+	}
+	db_insert(&mysql, querybuf);
 
 	/* cleanup and exit program */
 	pthread_attr_destroy(&attr);
 
-	SPINE_LOG_DEBUG(("DEBUG: Thread Cleanup Complete\n"));
+	SPINE_LOG_DEBUG(("DEBUG: Thread Cleanup Complete"));
 
 	/* close the php script server */
 	if (set.php_required) {
 		php_close(PHP_INIT);
 	}
 
-	SPINE_LOG_DEBUG(("DEBUG: PHP Script Server Pipes Closed\n"));
+	SPINE_LOG_DEBUG(("DEBUG: PHP Script Server Pipes Closed"));
 
 	/* free malloc'd variables */
 	free(threads);
 	free(ids);
 	free(conf_file);
 
-	SPINE_LOG_DEBUG(("DEBUG: Allocated Variable Memory Freed\n"));
-
-	/* shutdown SNMP */
-	snmp_spine_close();
-
-	SPINE_LOG_DEBUG(("SPINE: Net-SNMP API Shutdown Completed\n"));
+	SPINE_LOG_DEBUG(("DEBUG: Allocated Variable Memory Freed"));
 
 	/* close mysql */
 	mysql_free_result(result);
 	mysql_close(&mysql);
 
-	SPINE_LOG_DEBUG(("DEBUG: MYSQL Free & Close Completed\n"));
+	SPINE_LOG_DEBUG(("DEBUG: MYSQL Free & Close Completed"));
+
+	/* close snmp */
+	snmp_spine_close();
+
+	SPINE_LOG_DEBUG(("DEBUG: Net-SNMP Close Completed"));
 
 	/* finally add some statistics to the log and exit */
 	end_time = TIMEVAL_TO_DOUBLE(now);
 
 	if (set.log_level >= POLLER_VERBOSITY_MEDIUM) {
-		SPINE_LOG(("Time: %.4f s, Threads: %i, Hosts: %i\n", (end_time - begin_time), set.threads, num_rows));
+		SPINE_LOG(("Time: %.4f s, Threads: %i, Hosts: %i", (end_time - begin_time), set.threads, num_rows));
 	}else{
 		/* provide output if running from command line */
 		if (!set.stdout_notty) {
@@ -643,23 +842,23 @@ int main(int argc, char *argv[]) {
 static void display_help(void) {
 	static const char *const *p;
 	static const char * const helptext[] = {
-		"Usage: spine [options] [firstid lastid]",
+		"Usage: spine [options] [[firstid lastid] || [-H/--hostlist='hostid1,hostid2,...,hostidn']]",
 		"",
 		"Options:",
-		"",
 		"  -h/--help          Show this brief help listing",
-		"  -f/--first=X       Start polling with host X",
-		"  -l/--last=X        End polling with host X",
-		"  -p/--poller=X      Poller ID = X",
-		"  -C/--conf=F        Read Spine configuration from file F",
+		"  -f/--first=X       Start polling with host id X",
+		"  -l/--last=X        End polling with host id X",
+		"  -H/--hostlist=X    Poll the list of host ids, separated by comma's",
+		"  -p/--poller=X      Set the poller id to X",
+		"  -C/--conf=F        Read spine configuration from file F",
 		"  -O/--option=S:V    Override DB settings 'set' with value 'V'",
-		"  -R/--readonly      This Spine run is readonly with respect to the database",
-		"  -S/--stdout        Logging is performed to the standard output",
+		"  -R/--readonly      Spine will not write output to the DB",
+		"  -S/--stdout        Logging is performed to standard output",
 		"  -V/--verbosity=V   Set logging verbosity to <V>",
-		"  --snmponly         Only do SNMP polling: no script stuff",
+		"  --snmponly         Only do SNMP polling: no scripts",
 		"",
-		"Either both of --first/--last must be provided, or neither can be,",
-		"and in their absense, all hosts are processed.",
+		"Either both of --first/--last must be provided, a valid hostlist must be provided.",
+        "In their absence, all hosts are processed.",
 		"",
 		"Without the --conf parameter, spine searches for its spine.conf",
 		"file in the usual places.",
@@ -677,7 +876,7 @@ static void display_help(void) {
 		0 /* ENDMARKER */
 	};
 
-	printf("SPINE %s  Copyright 2002-2007 by The Cacti Group\n\n", VERSION);
+	printf("SPINE %s  Copyright 2002-2013 by The Cacti Group\n\n", VERSION);
 
 	for (p = helptext; *p; p++) {
 		puts(*p);	/* automatically adds a newline */
