@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2002-2007 The Cacti Group                                 |
+ | Copyright (C) 2002-2013 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -54,9 +54,7 @@ int db_insert(MYSQL *mysql, const char *query) {
 	snprintf(query_frag, BUFSIZE, "%s", query);
 
 	/* show the sql query */
-	if (set.log_level == 5) {
-		SPINE_LOG_DEBUG(("DEBUG: SQL:'%s'", query_frag));
-	}
+	SPINE_LOG_DEVDBG(("DEVDBG: SQL:'%s'", query_frag));
 
 	while(1) {
 		if (set.SQL_readonly == FALSE) {
@@ -68,13 +66,25 @@ int db_insert(MYSQL *mysql, const char *query) {
 					error_count++;
 
 					if (error_count > 30) {
-						SPINE_LOG(("ERROR: Too many Lock/Deadlock errors occurred!, SQL Fragment:'%s'\n", query_frag));
+						SPINE_LOG(("ERROR: Too many Lock/Deadlock errors occurred!, SQL Fragment:'%s'", query_frag));
 						return FALSE;
 					}
 
+					continue;
+				}else if (error == 2006) {
+					SPINE_LOG(("WARNING: SQL Failed! Error:'%i', Message:'%s', Attempting to Reconnect", error, mysql_error(mysql)));
+					db_disconnect(mysql);
+					usleep(50000);
+					db_connect(set.dbdb, mysql);
+					error_count++;
+
+					if (error_count > 30) {
+						die("FATAL: Too many Reconnect Attempts!\n");
+                                        }
+
 					continue;
 				}else{
-					SPINE_LOG(("ERROR: A database insert failed! Error:'%i', SQL Fragment:'%s'\n", error, query_frag));
+					SPINE_LOG(("ERROR: SQL Failed! Error:'%i', Message:'%s', SQL Fragment:'%s'", error, mysql_error(mysql), query_frag));
 					return FALSE;
 				}
 			}else{
@@ -108,7 +118,7 @@ MYSQL_RES *db_query(MYSQL *mysql, const char *query) {
 	snprintf(query_frag, BUFSIZE, "%s", query);
 
 	/* show the sql query */
-	SPINE_LOG_DEBUG(("DEBUG: SQL:'%s'", query_frag));
+	SPINE_LOG_DEVDBG(("DEVDBG: SQL:'%s'", query_frag));
 
 	while (1) {
 		if (mysql_query(mysql, query)) {
@@ -121,12 +131,24 @@ MYSQL_RES *db_query(MYSQL *mysql, const char *query) {
 				error_count++;
 
 				if (error_count > 30) {
-					SPINE_LOG(("ERROR: Too many Lock/Deadlock errors occurred!, SQL Fragment:'%s'\n", query_frag));
-
-					die("ERROR: Fatal MySQL Query Error, exiting!");
+					die("FATAL: Too many Lock/Deadlock errors occurred!, SQL Fragment:'%s'\n", query_frag);
 				}
 
 				continue;
+			}else if (error == 2006) {
+				SPINE_LOG(("WARNING: SQL Failed! Error:'%i', Message:'%s', Attempting to Reconnect", error, mysql_error(mysql)));
+				db_disconnect(mysql);
+				usleep(50000);
+				db_connect(set.dbdb, mysql);
+				error_count++;
+
+				if (error_count > 30) {
+					die("FATAL: Too many Reconnect Attempts!\n");
+				}
+
+				continue;
+			}else{
+				die("FATAL: MySQL Error:'%i', Message:'%s'", error, mysql_error(mysql));
 			}
 		}else{
 			mysql_res = mysql_store_result(mysql);
@@ -151,6 +173,8 @@ MYSQL_RES *db_query(MYSQL *mysql, const char *query) {
 void db_connect(const char *database, MYSQL *mysql) {
 	int    tries;
 	int    timeout;
+	int    rtimeout;
+	int    wtimeout;
 	int    options_error;
 	int    success;
 	char   *hostname;
@@ -177,29 +201,53 @@ void db_connect(const char *database, MYSQL *mysql) {
 	tries   = 5;
 	success = FALSE;
 	timeout = 5;
+	rtimeout = 10;
+	wtimeout = 20;
 
-	thread_mutex_lock(LOCK_MYSQL);
 	mysql_init(mysql);
 	if (mysql == NULL) {
 		die("FATAL: MySQL unable to allocate memory and therefore can not connect");
 	}
 
+	options_error = mysql_options(mysql, MYSQL_OPT_READ_TIMEOUT, (char *)&rtimeout);
+	if (options_error < 0) {
+		die("FATAL: MySQL options unable to set read timeout value");
+	}
+
+	options_error = mysql_options(mysql, MYSQL_OPT_WRITE_TIMEOUT, (char *)&wtimeout);
+	if (options_error < 0) {
+		die("FATAL: MySQL options unable to set read timeout value");
+	}
+
 	options_error = mysql_options(mysql, MYSQL_OPT_CONNECT_TIMEOUT, (char *)&timeout);
 	if (options_error < 0) {
-		die("FATAL: MySQL options unable to set timeout value\n");
+		die("FATAL: MySQL options unable to set timeout value");
 	}
 
+	#ifdef MYSQL_OPT_RECONNECT
+	my_bool reconnect = 1;
+	options_error = mysql_options(mysql, MYSQL_OPT_RECONNECT, &reconnect);
+	if (options_error < 0) {
+		die("FATAL: MySQL options unable to set reconnect option\n");
+	}
+	#endif
+
 	while (tries > 0) {
 		tries--;
 
 		if (!mysql_real_connect(mysql, hostname, set.dbuser, set.dbpass, database, set.dbport, socket, 0)) {
-			printf("MYSQL: Connection Failed: %s\n", mysql_error(mysql));
+			if ((mysql_errno(mysql) != 1049) && (mysql_errno(mysql) != 2005) && (mysql_errno(mysql) != 1045)) {
+				printf("MYSQL: Connection Failed: Error:'%u', Message:'%s'\n", mysql_errno(mysql), mysql_error(mysql));
 
-			success = FALSE;
+				success = FALSE;
 
-			#ifndef SOLAR_THREAD
-			usleep(2000);
-			#endif
+				#ifndef SOLAR_THREAD
+				usleep(2000);
+				#endif
+			}else{
+				tries   = 0;
+				success = FALSE;
+			}
 		}else{
 			tries   = 0;
 			success = TRUE;
@@ -208,10 +256,8 @@ void db_connect(const char *database, MYSQL *mysql) {
 
 	free(hostname);
 
-	thread_mutex_unlock(LOCK_MYSQL);
-
 	if (!success){
-		die("FATAL: Connection Failed: %s", mysql_error(mysql));
+		die("FATAL: Connection Failed, Error:'%i', Message:'%s'", mysql_errno(mysql), mysql_error(mysql));
 	}
 }
 
@@ -250,3 +296,24 @@ int append_hostrange(char *obuf, const char *colname) {
 		return 0;
 	}
 }
+
+/*! \fn void db_escape(MYSQL *mysql, char *output, const char *input)
+ *  \brief Escapse a text string to make it safe for mysql insert/updates
+ *  \param mysql the connection object
+ *  \param output a pointer to the output string
+ *  \param a pointer to the input string
+ *
+ *	A simple implementation of the mysql_real_escape_string that one
+ *  day should be portable.
+ *
+ *  \return void
+ *
+ */
+void db_escape(MYSQL *mysql, char *output, const char *input) {
+	if (input == NULL) return;
+
+	output[0] = '\0';
+
+	mysql_real_escape_string(mysql, output, input, strlen(input));
+}
+
