@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2002-2007 The Cacti Group                                 |
+ | Copyright (C) 2002-2012 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -73,8 +73,6 @@ void set_option(const char *option, const char *value) {
  *  NOTE: if the user has provided one of these options on the command line,
  *  it's intercepted here and returned, overriding the database setting.
  *
- *  ===TODO: use a prepared statement?
- *
  *  \return the database option setting
  *
  */
@@ -146,6 +144,45 @@ static int getboolsetting(MYSQL *psql, const char *setting, int dflt) {
 	return dflt;
 }
 
+/*! \fn static const char *getglobalvariable(MYSQL *psql, const char *setting)
+ *  \brief Returns a character pointer to a MySQL global variable setting.
+ *
+ *  Given a pointer to a database and the name of a global variable, return the string
+ *  which represents that value from the settings table. Return NULL if we
+ *  can't find a variable for whatever reason.
+ *
+ *  \return the database global variable setting
+ *
+ */
+static const char *getglobalvariable(MYSQL *psql, const char *setting) {
+	char      qstring[256];
+	MYSQL_RES *result;
+	MYSQL_ROW mysql_row;
+	int       i;
+
+	assert(psql    != 0);
+	assert(setting != 0);
+
+	/* see if it's in the option table */
+	for (i=0; i<nopts; i++) {
+		if (STRIMATCH(setting, opttable[i].opt)) {
+			/* FOUND IT! */
+			return opttable[i].val;
+		}
+	}
+
+	sprintf(qstring, "SHOW GLOBAL VARIABLES LIKE '%s'", setting);
+
+	result = db_query(psql, qstring);
+
+	if ((mysql_num_rows(result) > 0) &&
+		(mysql_row = mysql_fetch_row(result)) != 0)	{
+		return mysql_row[1];
+	}else{
+		return 0;
+	}
+}
+
 /*! \fn void read_config_options(void)
  *  \brief Reads the default Spine runtime parameters from the database and set's the global array
  *
@@ -162,6 +199,12 @@ void read_config_options() {
 
 	db_connect(set.dbdb, &mysql);
 
+	/* get the mysql server version */
+	set.dbversion = 0;
+	if ((res = getglobalvariable(&mysql, "version")) != 0 ) {
+		set.dbversion = atoi(res);
+	}
+
 	/* get logging level from database - overrides spine.conf */
 	if ((res = getsetting(&mysql, "log_verbosity")) != 0 ) {
 		const int n = atoi(res);
@@ -177,7 +220,7 @@ void read_config_options() {
 	/* determine logfile path */
 	if ((res = getsetting(&mysql, "path_cactilog")) != 0 ) {
 		if (strlen(res) != 0) {
-			snprintf(set.path_logfile, SMALL_BUFSIZE, res);
+			snprintf(set.path_logfile, SMALL_BUFSIZE, "%s", res);
 		}else{
 			if (strlen(web_root) != 0) {
 				snprintf(set.path_logfile, SMALL_BUFSIZE, "%s/log/cacti.log", web_root);
@@ -190,10 +233,10 @@ void read_config_options() {
  	}
 
 	/* log the path_webroot variable */
-	SPINE_LOG_DEBUG(("DEBUG: The path_php_server variable is %s\n", set.path_php_server));
+	SPINE_LOG_DEBUG(("DEBUG: The path_php_server variable is %s", set.path_php_server));
 
 	/* log the path_cactilog variable */
-	SPINE_LOG_DEBUG(("DEBUG: The path_cactilog variable is %s\n", set.path_logfile));
+	SPINE_LOG_DEBUG(("DEBUG: The path_cactilog variable is %s", set.path_logfile));
 
 	/* determine log file, syslog or both, default is 1 or log file only */
 	if ((res = getsetting(&mysql, "log_destination")) != 0 ) {
@@ -203,7 +246,7 @@ void read_config_options() {
 	}
 
 	/* log the log_destination variable */
-	SPINE_LOG_DEBUG(("DEBUG: The log_destination variable is %i (%s)\n",
+	SPINE_LOG_DEBUG(("DEBUG: The log_destination variable is %i (%s)",
 			set.log_destination,
 			printable_logdest(set.log_destination)));
 	set.logfile_processed = TRUE;
@@ -214,7 +257,7 @@ void read_config_options() {
 	}
 
 	/* log the path_php variable */
-	SPINE_LOG_DEBUG(("DEBUG: The path_php variable is %s\n", set.path_php));
+	SPINE_LOG_DEBUG(("DEBUG: The path_php variable is %s", set.path_php));
 
 	/* set availability_method */
 	if ((res = getsetting(&mysql, "availability_method")) != 0 ) {
@@ -222,7 +265,7 @@ void read_config_options() {
 	}
 
 	/* log the availability_method variable */
-	SPINE_LOG_DEBUG(("DEBUG: The availability_method variable is %i\n", set.availability_method));
+	SPINE_LOG_DEBUG(("DEBUG: The availability_method variable is %i", set.availability_method));
 
 	/* set ping_recovery_count */
 	if ((res = getsetting(&mysql, "ping_recovery_count")) != 0 ) {
@@ -230,7 +273,7 @@ void read_config_options() {
 	}
 
 	/* log the ping_recovery_count variable */
-	SPINE_LOG_DEBUG(("DEBUG: The ping_recovery_count variable is %i\n", set.ping_recovery_count));
+	SPINE_LOG_DEBUG(("DEBUG: The ping_recovery_count variable is %i", set.ping_recovery_count));
 
 	/* set ping_failure_count */
 	if ((res = getsetting(&mysql, "ping_failure_count")) != 0) {
@@ -238,7 +281,7 @@ void read_config_options() {
 	}
 
 	/* log the ping_failure_count variable */
-	SPINE_LOG_DEBUG(("DEBUG: The ping_failure_count variable is %i\n", set.ping_failure_count));
+	SPINE_LOG_DEBUG(("DEBUG: The ping_failure_count variable is %i", set.ping_failure_count));
 
 	/* set ping_method */
 	if ((res = getsetting(&mysql, "ping_method")) != 0 ) {
@@ -246,7 +289,7 @@ void read_config_options() {
 	}
 
 	/* log the ping_method variable */
-	SPINE_LOG_DEBUG(("DEBUG: The ping_method variable is %i\n", set.ping_method));
+	SPINE_LOG_DEBUG(("DEBUG: The ping_method variable is %i", set.ping_method));
 
 	/* set ping_retries */
 	if ((res = getsetting(&mysql, "ping_retries")) != 0 ) {
@@ -254,7 +297,7 @@ void read_config_options() {
 	}
 
 	/* log the ping_retries variable */
-	SPINE_LOG_DEBUG(("DEBUG: The ping_retries variable is %i\n", set.ping_retries));
+	SPINE_LOG_DEBUG(("DEBUG: The ping_retries variable is %i", set.ping_retries));
 
 	/* set ping_timeout */
 	if ( (res = getsetting(&mysql, "ping_timeout")) != 0 ) {
@@ -262,25 +305,39 @@ void read_config_options() {
 	}
 
 	/* log the ping_timeout variable */
-	SPINE_LOG_DEBUG(("DEBUG: The ping_timeout variable is %i\n", set.ping_timeout));
+	SPINE_LOG_DEBUG(("DEBUG: The ping_timeout variable is %i", set.ping_timeout));
+
+	/* set snmp_retries */
+	if ( (res = getsetting(&mysql, "snmp_retries")) != 0 ) {
+		set.snmp_retries = atoi(res);
+	}
+
+	/* log the snmp_retries variable */
+	SPINE_LOG_DEBUG(("DEBUG: The snmp_retries variable is %i", set.snmp_retries));
 
 	/* set logging option for errors */
 	set.log_perror = getboolsetting(&mysql, "log_perror", FALSE);
 
 	/* log the log_perror variable */
-	SPINE_LOG_DEBUG(("DEBUG: The log_perror variable is %i\n", set.log_perror));
+	SPINE_LOG_DEBUG(("DEBUG: The log_perror variable is %i", set.log_perror));
 
 	/* set logging option for errors */
 	set.log_pwarn = getboolsetting(&mysql, "log_pwarn", FALSE);
 
 	/* log the log_pwarn variable */
-	SPINE_LOG_DEBUG(("DEBUG: The log_pwarn variable is %i\n", set.log_pwarn));
+	SPINE_LOG_DEBUG(("DEBUG: The log_pwarn variable is %i", set.log_pwarn));
+
+	/* set logging option for errors */
+	set.boost_redirect = getboolsetting(&mysql, "boost_redirect", FALSE);
+
+	/* log the log_pwarn variable */
+	SPINE_LOG_DEBUG(("DEBUG: The boost_redirect variable is %i", set.boost_redirect));
 
 	/* set logging option for statistics */
 	set.log_pstats = getboolsetting(&mysql, "log_pstats", FALSE);
 
 	/* log the log_pstats variable */
-	SPINE_LOG_DEBUG(("DEBUG: The log_pstats variable is %i\n", set.log_pstats));
+	SPINE_LOG_DEBUG(("DEBUG: The log_pstats variable is %i", set.log_pstats));
 
 	/* get Cacti defined max threads override spine.conf */
 	if ((res = getsetting(&mysql, "max_threads")) != 0 ) {
@@ -291,7 +348,7 @@ void read_config_options() {
 	}
 
 	/* log the threads variable */
-	SPINE_LOG_DEBUG(("DEBUG: The threads variable is %i\n", set.threads));
+	SPINE_LOG_DEBUG(("DEBUG: The threads variable is %i", set.threads));
 
 	/* get the poller_interval for those who have elected to go with a 1 minute polling interval */
 	if ((res = getsetting(&mysql, "poller_interval")) != 0 ) {
@@ -302,12 +359,11 @@ void read_config_options() {
 
 	/* log the poller_interval variable */
 	if (set.poller_interval == 0) {
-		SPINE_LOG_DEBUG(("DEBUG: The polling interval is the system default\n"));
+		SPINE_LOG_DEBUG(("DEBUG: The polling interval is the system default"));
 	}else{
-		SPINE_LOG_DEBUG(("DEBUG: The polling interval is %i seconds\n", set.poller_interval));
+		SPINE_LOG_DEBUG(("DEBUG: The polling interval is %i seconds", set.poller_interval));
 	}
 
-
 	/* get the concurrent_processes variable to determine thread sleep values */
 	if ((res = getsetting(&mysql, "concurrent_processes")) != 0 ) {
 		set.num_parent_processes = atoi(res);
@@ -316,7 +372,7 @@ void read_config_options() {
 	}
 
 	/* log the concurrent processes variable */
-	SPINE_LOG_DEBUG(("DEBUG: The number of concurrent processes is %i\n", set.num_parent_processes));
+	SPINE_LOG_DEBUG(("DEBUG: The number of concurrent processes is %i", set.num_parent_processes));
 
 	/* get the script timeout to establish timeouts */
 	if ((res = getsetting(&mysql, "script_timeout")) != 0 ) {
@@ -329,7 +385,7 @@ void read_config_options() {
 	}
 
 	/* log the script timeout value */
-	SPINE_LOG_DEBUG(("DEBUG: The script timeout is %i\n", set.script_timeout));
+	SPINE_LOG_DEBUG(("DEBUG: The script timeout is %i", set.script_timeout));
 
 	/* get the number of script server processes to run */
 	if ((res = getsetting(&mysql, "php_servers")) != 0 ) {
@@ -347,7 +403,7 @@ void read_config_options() {
 	}
 
 	/* log the script timeout value */
-	SPINE_LOG_DEBUG(("DEBUG: The number of php script servers to run is %i\n", set.php_servers));
+	SPINE_LOG_DEBUG(("DEBUG: The number of php script servers to run is %i", set.php_servers));
 
 	/*----------------------------------------------------------------
 	 * determine if the php script server is required by searching for
@@ -358,24 +414,47 @@ void read_config_options() {
 	 */
 	set.php_required = FALSE;		/* assume no */
 
-	sqlp = sqlbuf;
-	sqlp += sprintf(sqlp, "SELECT action FROM poller_item");
-	sqlp += sprintf(sqlp, " WHERE action=%d", POLLER_ACTION_PHP_SCRIPT_SERVER);
-	sqlp += append_hostrange(sqlp, "host_id");
-	sqlp += sprintf(sqlp, " LIMIT 1");
-
-	result = db_query(&mysql, sqlbuf);
-	num_rows = (int)mysql_num_rows(result);
-
-	if (num_rows > 0) set.php_required = TRUE;
-
 	/* log the requirement for the script server */
-	SPINE_LOG_DEBUG(("DEBUG: StartHost='%i', EndHost='%i', TotalPHPScripts='%i'\n",
-		set.start_host_id,
-		set.end_host_id,
-		num_rows));
+	if (!strlen(set.host_id_list)) {
+		sqlp = sqlbuf;
+		sqlp += sprintf(sqlp, "SELECT action FROM poller_item");
+		sqlp += sprintf(sqlp, " WHERE action=%d", POLLER_ACTION_PHP_SCRIPT_SERVER);
+		sqlp += append_hostrange(sqlp, "host_id");
+		if (set.poller_id_exists) {
+			sqlp += sprintf(sqlp, " AND poller_id=%i", set.poller_id);
+		}
+		sqlp += sprintf(sqlp, " LIMIT 1");
 
-	SPINE_LOG_DEBUG(("DEBUG: The PHP Script Server is %sRequired\n",
+		result = db_query(&mysql, sqlbuf);
+		num_rows = mysql_num_rows(result);
+
+		if (num_rows > 0) set.php_required = TRUE;
+
+		SPINE_LOG_DEBUG(("DEBUG: StartHost='%i', EndHost='%i', TotalPHPScripts='%i'",
+			set.start_host_id,
+			set.end_host_id,
+			num_rows));
+	}else{
+		sqlp = sqlbuf;
+		sqlp += sprintf(sqlp, "SELECT action FROM poller_item");
+		sqlp += sprintf(sqlp, " WHERE action=%d", POLLER_ACTION_PHP_SCRIPT_SERVER);
+		sqlp += sprintf(sqlp, " AND host_id IN(%s)", set.host_id_list);
+		if (set.poller_id_exists) {
+			sqlp += sprintf(sqlp, " AND poller_id=%i", set.poller_id);
+		}
+		sqlp += sprintf(sqlp, " LIMIT 1");
+
+		result = db_query(&mysql, sqlbuf);
+		num_rows = mysql_num_rows(result);
+
+		if (num_rows > 0) set.php_required = TRUE;
+
+		SPINE_LOG_DEBUG(("DEBUG: Host List to be polled='%s', TotalPHPScripts='%i'",
+			set.host_id_list,
+			num_rows));
+	}
+
+	SPINE_LOG_DEBUG(("DEBUG: The PHP Script Server is %sRequired",
 		set.php_required
 		? ""
 		: "Not "));
@@ -392,7 +471,7 @@ void read_config_options() {
 	}
 
 	/* log the snmp_max_get_size variable */
-	SPINE_LOG_DEBUG(("DEBUG: The Maximum SNMP OID Get Size is %i\n", set.snmp_max_get_size));
+	SPINE_LOG_DEBUG(("DEBUG: The Maximum SNMP OID Get Size is %i", set.snmp_max_get_size));
 
 	mysql_free_result(result);
 	db_disconnect(&mysql);
@@ -407,6 +486,7 @@ void read_config_options() {
 int read_spine_config(char *file) {
 	FILE *fp;
 	char buff[BUFSIZE];
+	char *buffer;
 	char p1[BUFSIZE];
 	char p2[BUFSIZE];
 
@@ -423,15 +503,18 @@ int read_spine_config(char *file) {
 		}
 
 		while(!feof(fp)) {
-			fgets(buff, BUFSIZE, fp);
+			buffer = fgets(buff, BUFSIZE, fp);
 			if (!feof(fp) && *buff != '#' && *buff != ' ' && *buff != '\n') {
 				sscanf(buff, "%15s %255s", p1, p2);
 
-				if (STRIMATCH(p1, "DB_Host"))          STRNCOPY(set.dbhost, p2);
-				else if (STRIMATCH(p1, "DB_Database")) STRNCOPY(set.dbdb, p2);
-				else if (STRIMATCH(p1, "DB_User"))     STRNCOPY(set.dbuser, p2);
-				else if (STRIMATCH(p1, "DB_Pass"))     STRNCOPY(set.dbpass, p2);
-				else if (STRIMATCH(p1, "DB_Port"))     set.dbport = atoi(p2);
+				if (STRIMATCH(p1, "DB_Host"))               STRNCOPY(set.dbhost, p2);
+				else if (STRIMATCH(p1, "DB_Database"))      STRNCOPY(set.dbdb, p2);
+				else if (STRIMATCH(p1, "DB_User"))          STRNCOPY(set.dbuser, p2);
+				else if (STRIMATCH(p1, "DB_Pass"))          STRNCOPY(set.dbpass, p2);
+				else if (STRIMATCH(p1, "DB_Port"))          set.dbport    = atoi(p2);
+				else if (STRIMATCH(p1, "Poller"))           set.poller_id = atoi(p2);
+				else if (STRIMATCH(p1, "DB_PreG"))          set.pre087g   = atoi(p2);
+				else if (STRIMATCH(p1, "SNMP_Clientaddr"))  STRNCOPY(set.snmp_clientaddr, p2);
 				else if (!set.stderr_notty) {
 					fprintf(stderr,"WARNING: Unrecongized directive: %s=%s in %s\n", p1, p2, file);
 				}
@@ -454,7 +537,8 @@ int read_spine_config(char *file) {
  */
 void config_defaults() {
 	set.threads = DEFAULT_THREADS;
-	set.dbport = DEFAULT_DB_PORT;
+	set.dbport  = DEFAULT_DB_PORT;
+	set.pre087g = DEFAULT_DB_PREG;
 
 	STRNCOPY(set.dbhost, DEFAULT_DB_HOST);
 	STRNCOPY(set.dbdb,   DEFAULT_DB_DB  );
@@ -464,6 +548,7 @@ void config_defaults() {
 	STRNCOPY(config_paths[0], CONFIG_PATH_1);
 	STRNCOPY(config_paths[1], CONFIG_PATH_2);
 	STRNCOPY(config_paths[2], CONFIG_PATH_3);
+	STRNCOPY(config_paths[3], CONFIG_PATH_4);
 
 	set.log_destination = LOGDEST_FILE;
 }
@@ -494,7 +579,7 @@ void die(const char *format, ...) {
 		snprintf(flogmessage, BUFSIZE, "%s (Spine init)", logmessage);
 	}
 
-	SPINE_LOG((flogmessage));
+	SPINE_LOG(("%s", flogmessage));
 
 	if (set.parent_fork == SPINE_PARENT) {
 		if (set.php_initialized) {
@@ -535,6 +620,11 @@ int spine_log(const char *format, ...) {
 	/* default for "console" messages to go to stdout */
 	fp = stdout;
 
+	if (IS_LOGGING_TO_STDOUT()) {
+		puts(trim(ulogmessage));
+		return TRUE;
+	}
+
 	/* append a line feed to the log message if needed */
 	if (!strstr(ulogmessage, "\n")) {
 		strncat(ulogmessage, "\n", 1);
@@ -543,11 +633,6 @@ int spine_log(const char *format, ...) {
 	/* log message prefix */
 	snprintf(logprefix, SMALL_BUFSIZE, "SPINE: Poller[%i] ", set.poller_id);
 
-	if (IS_LOGGING_TO_STDOUT()) {
-		puts(ulogmessage);
-		return TRUE;
-	}
-
 	/* get time for poller_output table */
 	nowbin = time(&nowbin);
 
@@ -573,8 +658,8 @@ int spine_log(const char *format, ...) {
 	strncat(flogmessage, logprefix,   sizeof(flogmessage)-1);
 	strncat(flogmessage, ulogmessage, sizeof(flogmessage)-1);
 
-	if ((IS_LOGGING_TO_FILE() && 
-		(set.log_level != POLLER_VERBOSITY_NONE) && 
+	if ((IS_LOGGING_TO_FILE() &&
+		(set.log_level != POLLER_VERBOSITY_NONE) &&
 		(strlen(set.path_logfile) != 0))) {
 		if (set.logfile_processed) {
 			if (!file_exists(set.path_logfile)) {
@@ -670,12 +755,38 @@ int all_digits(const char *string) {
 	/* empty string is not all digits */
 	if ( *string == '\0' ) return FALSE;
 
-	while ( isdigit(*string) )
+	while ( isdigit((int)*string) )
 		string++;
 
 	return *string == '\0';
 }
 
+/*! \fn is_ipaddress(const char *string)
+ *  \brief verifies that a string is an ip address either v4 or v6
+ *  \param string the string to check
+ *
+ *  This function simply checks to see if a string object is an ip address.
+ *  If it is, it returns true else false.
+ *
+ *  \return TRUE if an ip address, or FALSE if non
+ *
+ */
+int is_ipaddress(const char *string) {
+	while (*string) {
+		if ((isdigit((int)*string)) ||
+			(*string == '.') ||
+			(*string == ':')) {
+			string++;
+
+			continue;
+		}
+
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 /*! \fn int is_numeric(const char *string)
  *  \brief check to see if a string is long or double
  *  \param string the string to check
@@ -683,8 +794,7 @@ int all_digits(const char *string) {
  *  \return TRUE if long or double, FALSE if not
  *
  */
-int is_numeric(const char *string)
-{
+int is_numeric(const char *string) {
 	long local_lval;
 	double local_dval;
 	char *end_ptr_long, *end_ptr_double;
@@ -715,6 +825,7 @@ int is_numeric(const char *string)
 		end_ptr_long = NULL;
 	}
 
+ 	/* check for a float */
 	errno = 0;
 	local_dval = strtod(string, &end_ptr_double);
 	if (errno != ERANGE) {
@@ -726,27 +837,67 @@ int is_numeric(const char *string)
 	}
 
 	if (!errno) {
-		return TRUE;
+		if (STRIMATCH(string," ")) {
+			return FALSE;
+		}else{
+			return TRUE;
+		}
 	}else{
 		return FALSE;
- 	}
+	}
+}
+
+/*! \fn int is_hexadecimal(const char *str, const short ignore_space)
+ *  \brief test whether a string represents a hex number.
+ *  \param str string to test
+ *  \param ignore_space nonzero to skip tabs and spaces
+ *
+ *  \return TRUE if the string is valid hex, FALSE otherwise
+ *
+ */
+int is_hexadecimal(const char * str, const short ignore_space) {
+	int i = 0;
+
+	if (!str) return FALSE;
+
+	while (*str) {
+		switch (*str) {
+		case '0': case '1': case '2': case '3':
+		case '4': case '5': case '6': case '7':
+		case '8': case '9':
+		case 'a': case 'A': case 'b': case 'B':
+		case 'c': case 'C': case 'd': case 'D':
+		case 'e': case 'E': case 'f': case 'F':
+		case '"':
+			break;
+		case ' ': case '\t':
+			if (ignore_space) break;
+		default:
+			return FALSE;
+		}
+		str++;
+		i++;
+	}
+
+	if (i < 3) return FALSE;
+
+	return TRUE;
 }
 
 /*! \fn char *strip_alpha(char *string)
  *  \brief remove trailing alpha characters from a string.
- *  \param string the string to string characters from
+ *  \param string the string to strip characters from
  *
  *  \return a pointer to the modified string
  *
  */
-char *strip_alpha(char *string)
-{
+char *strip_alpha(char *string) {
 	int i;
 
 	i = strlen(string);
 
 	while (i >= 0) {
-		if (isdigit(string[i])) {
+		if (isdigit((int)string[i])) {
 			break;
 		}else{
 			string[i] = '\0';
@@ -757,17 +908,15 @@ char *strip_alpha(char *string)
 	return string;
 }
 
-/*! \fn char *add_slashes(char *string, int arguments_2_strip)
- *  \brief change all backslashes to forward slashes for the first n arguements.
+/*! \fn char *add_slashes(char *string)
+ *  \brief add escaping to back slashes on for Windows type commands.
  *  \param string the string to replace slashes
- *  \param arguments_2_strip the number of space delimited arguments to reverse
  *
  *  \return a pointer to the modified string. Variable must be freed by parent.
  *
  */
-char *add_slashes(char *string, int arguments_2_strip) {
+char *add_slashes(char *string) {
 	int length;
-	int space_count;
 	int position;
 	int new_position;
 	char *return_str;
@@ -777,10 +926,9 @@ char *add_slashes(char *string, int arguments_2_strip) {
 	}
 	return_str[0] = '\0';
 
-	length = strlen(string);
-	space_count = 0;
-	position = 0;
-	new_position = position;
+	length       = strlen(string);
+	position     = 0;
+	new_position = 0;
 
 	/* simply return on blank string */
 	if (!length) {
@@ -790,17 +938,9 @@ char *add_slashes(char *string, int arguments_2_strip) {
 	while (position < length) {
 		/* backslash detected, change to forward slash */
 		if (string[position] == '\\') {
-			/* only add slashes for first x arguments */
-			if (space_count < arguments_2_strip) {
-				return_str[new_position] = '/';
-			}else{
-				return_str[new_position] = string[position];
-			}
-		/* end of argument detected */
-		}else if (string[position] == ' ') {
-			return_str[new_position] = ' ';
-			space_count++;
-		/* normal character detected */
+			return_str[new_position] = '\\';
+			new_position++;
+			return_str[new_position] = '\\';
 		}else{
 			return_str[new_position] = string[position];
 		}
@@ -812,81 +952,6 @@ char *add_slashes(char *string, int arguments_2_strip) {
 	return(return_str);
 }
 
-/*! \fn char *strip_string_crlf(char *string)
- *  \brief remove trailing cr-lf from a string
- *  \param string the string that requires trimming
- *
- *  \return a pointer to the modified string.
- *
- */
-char *strip_string_crlf(char *string) {
-	char *posptr;
-
-	posptr = strchr(string,'\n');
-
-	while(posptr != NULL) {
-		*posptr = '\0';
-		posptr = strchr(string,'\n');
-	}
-
-	posptr = strchr(string,'\r');
-
-	while(posptr != NULL) {
-		*posptr = '\0';
-		posptr = strchr(string,'\r');
-	}
-
-	return(string);
-}
-
-/*! \fn char *strip_quotes(char *string)
- *  \brief remove single and double quotes from a string
- *  \param string the string that requires trimming
- *
- *	Some SNMP agents return strings surrounded with single or double quotes,
- *	and we need to strip these off; We remove only *leading and trailing*
- *	quotes, not intermediate ones.
- *
- *  \return a pointer to the modified string.
- *
- */
-char *strip_quotes(char *string) {
-	int length;
-	char *startptr;
-	char type;
-
-	/* find first quote in the string, determine type */
-	while (1) {
-		length = strlen(string);
-
-		/* simply return on blank string */
-		if (!length) {
-			return string;
-		}
-
-		/* set starting postion of string */
-		startptr = string;
-
-		/* search for quote characters and remove */
-		if (string[0] == '"') {
-			type = '"';
-			memmove(startptr, startptr+1, strlen(string) - 1);
-		}else if (string[0] == '\'') {
-			type = '\'';
-			memmove(startptr, startptr+1, strlen(string) - 1);
-		}else if (string[0] == '\\') {
-			type = '\\';
-			memmove(startptr, startptr+1, strlen(string) - 1);
-		}else{
-			break;
-		}
-
-		string[length-1] = '\0';
-	}
-
-	return string;
-}
-
 /*! \fn char *strncopy(char *dst, const char *src, size_t obuf)
  *  \brief copies source to destination add a NUL terminator
  *
@@ -908,7 +973,7 @@ char *strncopy(char *dst, const char *src, size_t obuf) {
 	assert(dst != 0);
 	assert(src != 0);
 
-	int len;
+	size_t len;
 
 	len = strlen(src);
 
@@ -970,4 +1035,268 @@ char *get_host_poll_time() {
 	return(host_time);
 }
 
+/*! \fn trim()
+ *  \brief removes leading and trailing blanks, tabs, line feeds and
+ *         carriage returns from a string.
+ *
+ *  \return the trimmed string.
+ */
+char *trim(char *str) {
+	return ltrim(rtrim(str));
+}
 
+/*! \fn rtrim()
+ *  \brief removes trailing blanks, tabs, line feeds, carriage returns
+ *         single and double quotes and back-slashed from a string.
+ *
+ *  \return the trimmed string.
+ */
+char *rtrim(char *str) {
+	char    *end;
+	char    *trim = " \"\'\\\t\n\r";
+
+	if (!str) return NULL;
+
+	end = str + strlen(str);
+
+	while (end-- > str) {
+		if (!strchr(trim, *end)) return str;
+
+		*end = 0;
+	}
+
+	return str;
+}
+
+/*! \fn ltrim()
+ *  \brief removes leading blanks, tabs, line feeds, carriage returns
+ *         single and double quotes and back-slashed from a string.
+ *
+ *  \return the trimmed string.
+ */
+char *ltrim(char *str) {
+	char    *trim = " \"\'\\\t\n\r";
+
+	if (!str) return NULL;
+
+	while (*str) {
+		if (!strchr(trim, *str)) return str;
+
+		++str;
+	}
+
+	return str;
+}
+
+/*! \fn reverse()
+ *  \brief reverses a string in place.
+ *
+ *  \return the reversed string.
+ */
+char *reverse(char* str) {
+	int end   = strlen(str)-1;
+	int start = 0;
+
+	while (start < end) {
+		str[start] ^= str[end];
+		str[end]   ^= str[start];
+		str[start] ^= str[end];
+
+		++start;
+		--end;
+	}
+
+	return str;
+}
+
+/*! \fn char_count()
+ *  \brief counts occurrences of char in string.
+ *
+ *  \return number of occurrences.
+ */
+int char_count(const char *str, int chr) {
+	const unsigned char *my_str = (const unsigned char *) str;
+	const unsigned char my_chr = chr;
+	int count = 0;
+
+	if (!my_chr) return 1;
+
+	while (*my_str) {
+		if (*my_str++ == my_chr) {
+			count++;
+		}
+	}
+	return count;
+}
+
+unsigned long long hex2dec(char *str) {
+	int i = 0;
+	unsigned long long number = 0;
+	
+	if (!str) return 0;
+
+	/* first revers the string */
+	reverse(str);
+	
+	while (*str) {
+		switch (*str) {
+		case '0': 
+			i++;
+			break;
+		case '1': 
+			number += pow(16, i) * 1;
+			i++;
+			break;
+		case '2': 
+			number += pow(16, i) * 2;
+			i++;
+			break;
+		case '3':
+			number += pow(16, i) * 3;
+			i++;
+			break;
+		case '4': 
+			number += pow(16, i) * 4;
+			i++;
+			break;
+		case '5': 
+			number += pow(16, i) * 5;
+			i++;
+			break;
+		case '6': 
+			number += pow(16, i) * 6;
+			i++;
+			break;
+		case '7':
+			number += pow(16, i) * 7;
+			i++;
+			break;
+		case '8': 
+			number += pow(16, i) * 8;
+			i++;
+			break;
+		case '9':
+			number += pow(16, i) * 9;
+			i++;
+			break;
+		case 'a': case 'A': 
+			number += pow(16, i) * 10;
+			i++;
+			break;
+		case 'b': case 'B':
+			number += pow(16, i) * 11;
+			i++;
+			break;
+		case 'c': case 'C': 
+			number += pow(16, i) * 12;
+			i++;
+			break;
+		case 'd': case 'D':
+			number += pow(16, i) * 13;
+			i++;
+			break;
+		case 'e': case 'E': 
+			number += pow(16, i) * 14;
+			i++;
+			break;
+		case 'f': case 'F':
+			number += pow(16, i) * 15;
+			i++;
+			break;
+		case '"': case ' ': case '\t':
+			break;
+		default:
+			return 0;
+		}
+
+		str++;
+	}
+
+	return number;
+}
+
+int hasCaps() {
+	#ifdef HAVE_LCAP
+	cap_t caps;
+	cap_value_t capval;
+	cap_flag_value_t capflag;
+
+	caps = cap_from_text("cap_net_raw=eip");
+	if (caps == NULL) {
+		SPINE_LOG_DEBUG(("WARNING: cap_from_text failed."));
+		return FALSE;
+	}
+
+	for (capval=0; ;capval++) {
+		if (cap_get_flag(caps, capval, CAP_EFFECTIVE, &capflag))
+			break;
+		if (capflag != CAP_SET)
+			return FALSE;
+	}
+	return TRUE;
+	#else
+	return FALSE;
+	#endif
+}
+	
+void checkAsRoot() {
+	#ifndef __CYGWIN__
+	#ifdef SOLAR_PRIV
+	priv_set_t *privset;
+	char *p;
+
+	/* Get the basic set */
+	privset = priv_str_to_set("basic", ",", NULL);
+	if (privset == NULL) {
+		die("ERROR: Could not get basic privset from priv_str_to_set().");
+	} else {
+		p = priv_set_to_str(privset, ',', 0);
+		SPINE_LOG_DEBUG(("DEBUG: Basic privset is: '%s'.", p != NULL ? p : "Unknown"));
+	}
+
+	/* Add priviledge to send/receive ICMP packets */
+	if (priv_addset(privset, PRIV_NET_ICMPACCESS) < 0 ) {
+		SPINE_LOG_DEBUG(("Warning: Addition of PRIV_NET_ICMPACCESS to privset failed: '%s'.", strerror(errno)));
+	}
+
+	/* Compute the set of privileges that are never needed */
+	priv_inverse(privset);
+
+	/* Remove the set of unneeded privs from Permitted (and by
+	 * implication from Effective) */
+	if (setppriv(PRIV_OFF, PRIV_PERMITTED, privset) < 0) {
+		SPINE_LOG_DEBUG(("Warning: Dropping privileges from PRIV_PERMITTED failed: '%s'.", strerror(errno)));
+	}
+
+	/* Remove unneeded priv set from Limit to be safe */
+	if (setppriv(PRIV_OFF, PRIV_LIMIT, privset) < 0) {
+		SPINE_LOG_DEBUG(("Warning: Dropping privileges from PRIV_LIMIT failed: '%s'.", strerror(errno)));
+	}
+
+	boolean_t pe = priv_ineffect(PRIV_NET_ICMPACCESS);
+	SPINE_LOG_DEBUG(("DEBUG: Privilege PRIV_NET_ICMPACCESS is: '%s'.", pe != 0 ? "Enabled" : "Disabled"));
+
+	set.icmp_avail = pe;
+
+	/* Free the privset */
+	priv_freeset(privset);
+	free(p);
+	#else
+	if (hasCaps() != TRUE) {
+		seteuid(0);
+	
+		if (geteuid() != 0) {
+			SPINE_LOG_DEBUG(("WARNING: Spine NOT running asroot.  This is required if using ICMP.  Please run \"chmod +s;chown root:root spine\" to resolve."));
+			set.icmp_avail = FALSE;
+		}else{
+			SPINE_LOG_DEBUG(("DEBUG: Spine is running asroot."));
+			set.icmp_avail = TRUE;
+			seteuid(getuid());
+		}
+	} else {
+		SPINE_LOG_DEBUG(("DEBUG: Spine has cap_net_raw capability."));
+		set.icmp_avail = TRUE;
+	}
+	#endif
+	#endif
+}
